/* colmain.skel Skeleton for collection main program
 * $Log: colmain.skel,v $
 * Revision 1.20  1995/10/18  02:05:30  nort
 * Changes to nlcon_display()
 *
 * Revision 1.19  1995/10/06  16:40:02  nort
 * Changes to nlcon_disp()
 * Old change to ROWS() and FRACSECS() macros
 *
 * Revision 1.18  1994/08/29  20:29:15  nort
 * Added check for obsolete MSG_LABEL
 *
 * Revision 1.17  1994/08/29  20:15:12  nort
 * Mods for OUI: Moved several initializations into oui_init_options()
 * Made OPT_COL_INIT Col_init_options() and CONSOLE_INIT obsolete
 * Changed calls to msg() to nl_error(), making TMC collection independent
 * of the msg module, since nl_error is made to be redefined.
 *
 * Revision 1.16  1994/02/15  05:56:10  nort
 * Added some verbosity to COL_get_pointer()
 * I don't know why seteuid(getuid()) has been commented out:
 * I should try it again later.
 *
 * Revision 1.15  1993/09/27  18:53:20  nort
 * Added seteuid(getuid())
 * Cleaned up RCS log
 * Added cmrcsid[]
 * Added include <string.h> for memcpy prototype
 *
 * Revision 1.14  1993/03/08  18:11:45  nort
 * Added timerbd timing when available
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/kernel.h>
#include <sys/timers.h>
#include <sys/proxy.h>
#include <time.h>
#include <unistd.h>
#include "nortlib.h"
#include "oui.h"
#include "dbr.h"
#include "msg.h"
#include "globmsg.h"
#include "cc.h"
#include "collect.h"
#include "timerbd.h"
#pragma off (unreferenced)
  static char cmrcsid[] =
	"$Id: colmain.skel,v 1.20 1995/10/18 02:05:30 nort Exp nort $";
#pragma on (unreferenced)



 typedef unsigned short UINT;


 typedef unsigned char UCHAR;
	unsigned short InjectingInc = 0;


 typedef unsigned long ULONG;
ULONG CHROMtime;
#include "convfunc.h"
#include "ttdriver.h"
#include "defcals.h"

int tt_p4=72;
int tt_p3=48;
int tt_p2=24;
int tt_p1=0;




	
	

	 typedef float adT;
	 typedef float adT_flow;
	 typedef float vdc_chan;
	 typedef float vdc_pow;
	 typedef float adT_pump;

	 typedef float dacMSpr;

	 typedef UINT InjStateT;

float read_atod(int atod_chan) {
	
	unsigned short atod_val;
	float voltage = -99999;

	atod_val = tt_read_atod( atod_chan );
	atod_val = atod_val + 2048;
	atod_val = atod_val & 0xFFF;

	if (atod_val >= 2048)	{
		atod_val = atod_val|(0xF000);
		atod_val=~atod_val;
		++atod_val;
		voltage = -10.0 * ((float) atod_val) / 2048.0;
		//	printf("reading atod channel %i value is negative %i \n",atod_chan, atod_val);
	} else 	{
		voltage = 10.0 * ((float) atod_val) / 2048.0;
	//	printf("reading atod channel %i value is %f \n",atod_chan,voltage);

	}	

	return voltage;
}


float read_temp(int address) { 
	float voltage;

	voltage = read_atod(address);

	return convert_temps(voltage);
}


float read_temp12v(int address) { 
	float voltage;

	voltage = read_atod(address);

	return convert_temps12v(voltage);
}

float read_ad_value(int address, float R1, float V1, float V0) { 
	float Vx;

	Vx = read_atod(address);

	if (Vx < 0)
		Vx = -Vx;

	return convert_VtoR( Vx, R1, V1, V0 );

}



#include "valves.h"
#include "address.h"





	 typedef unsigned char DigOutWd;
	 typedef DigOutWd DigInWd;



	 typedef short DigBit;
	 typedef DigBit Bit_HI_LO;
	 typedef DigBit Bit_Op_Cl;
	 typedef DigBit Bit_Cl_Op;
	 typedef DigBit Bit_On_Off;
	 typedef DigBit Bit_Off_On;
	 typedef DigBit Bit_Air_Cal;
	 typedef DigBit Bit_offline;


	UCHAR MSDstateS;

int read_dig_in (unsigned short dig_chan)  {
	
	unsigned short dig_val;
	unsigned short address;
	int value;

	address=dig_chan/8;	
	dig_val=tt_read_digital(address);
	address=dig_chan-address*8;
	value=1<<address;
	//	address=pow(2,address);
	dig_val=dig_val&value;
	if (dig_val==0)
		return 0;
	return 1;
}

int read_dig_out(int chan) {

	return read_dig_in(chan+64);
}


int fail_board(int digch) {
	static int fail_toggle = 0;
	
	if (fail_toggle==0) {
		tt_scdc_command(100 + digch);
		fail_toggle = 1;
		return fail_toggle;
	}

	fail_toggle = 0;
	tt_scdc_command(digch);
	return fail_toggle;
}

// used to pulse a digch (hard coded for dig 20 and 21
// and MSDstate 4 and 5).
int trapToggle( int digch ) {
	static int pulse1 = 0;
	static int pulse2 = 0;
	int	pulse, state;
	const int trap1add = 20;
	const int trap2add = 21;

	if (digch == trap1add) {
	 	state = 4;
		pulse = pulse1;
	} else if (digch == trap2add) {
		state = 5;
		pulse = pulse2;
	} else {
		return 0;
	}

	// toggle dig line
	if ( MSDstateS == state ) {
		switch ( pulse ) {
			case 0:		// off, turn on
				tt_scdc_command( 100 + digch );
				pulse = 1;
				break;
			case 1:		// on, turn off
				tt_scdc_command( digch );
				pulse = 0;
				break;
		}
	} else {
		if ( pulse == 1 ) {
			tt_scdc_command( digch );
			pulse = 0;
		}
	}

	if (digch == trap1add)
		pulse1 = pulse;
	else
		pulse2 = pulse;

	return read_dig_out( digch );

}


#include <sys/types.h>
pid_t rs485_proxy;
short Omega485[34];

short OmegaArray( int add ) {
	return Omega485[add];
}



 typedef short OmegaT;

 typedef short OmegaTsp;
#include <math.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/types.h>
#include <stdlib.h>
#include "/home/panther/drivers/smart/smart.h"
#include "regnames.h"
#include "panthermsg.h"

#define 		ECDOFFLINE 		-9999
#define			NUMSECSXAXIS	 150
#define			ELECRATE		0.1			// seconds

pid_t			ECDresp_proxy;

chrom_packet 	ECDresp[MAXDET];

long int 		DetAvgCt1=0, DetAvgCt2=0, DetAvgCt3=0, DetAvgCt4=0,
		 		DetAvgOffline1=0, DetAvgOffline2=0, DetAvgOffline3=0, DetAvgOffline4=0,
	 			ecd_sum1=0, ecd_sum2=0, ecd_sum3=0, ecd_sum4=0,
				ecdA_SF6save=0, ecdA_F11save=0, ecdA_COsave=0, ecdA_PANsave=0;
float			ecdSF6x = 0.0, ecdF11x = 0.0, ecdCOx = 0.0, ecdPANx = 0.0;
int				currPoint[MAXDET];

// function gets the data from shared data array with smart driver
long int GC_Ctr_Pull( int elecADD ) {
	long ecdDATA;
	
	if ( currPoint[elecADD] >= MAXPOINTS ) currPoint[elecADD] = 0;

	// new data
	if ( currPoint[elecADD] < ECDresp[elecADD].numpnts ) {
		ecdDATA = ECDresp[elecADD].resp[currPoint[elecADD]];
		currPoint[elecADD]++;
	} 
	else {
		ecdDATA = ECDOFFLINE;
		currPoint[elecADD] = 0;
		ECDresp[elecADD].numpnts = -1;
	}

	return ecdDATA;	
}




 typedef long GC_Freq;
 typedef long GC_Ctr;

GC_Freq ecdA_SF6, ecdA_F11, ecdA_CO, ecdA_PAN;

 typedef unsigned char UNYBL;


 typedef unsigned long InjFile;

InjFile ITXFileNum;




ULONG CUMSEC;
#include <sys/types.h>
pid_t valco_proxy;
int valvePos[10] = {0,0,3,3,3,3,3,3,3,3};


 typedef short valvT;
 typedef DigBit valvetext;
 typedef DigBit valveMS1text;
 typedef DigBit valveMS2text;
#include <math.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "da_cache.h"
#include "regnames.h"

#define MAXMSDCHANS 	  8		// Max number of MS channels
#define MAXMSDIONS 		 30		// Max number of ions per channel
#define MAXMSDPOINTS 	 30		// Max Number of data points in a second for an ion
#define MSDOFFLINE 	  -9999
#define MSDMISSING	   -999
#define MAXSENDMSG     2000		// max number of chars from the msd server
#define SEPCHARS		":,()"
#define MSDIONFILE		"/home/panther/flight/msd.ions"

pid_t		MSDresp_proxy;

long int 	MSD_Data_Array[MAXMSDCHANS][MAXMSDPOINTS],
			MSD_Time_Array[MAXMSDPOINTS],
			msdTIMEoldPlot,
			MSchanAvg[MAXMSDCHANS],
			msd_sum[MAXMSDCHANS],
			MSD_ChanX;

unsigned int 	currMSDPoint[MAXMSDCHANS],
				chanInc;

unsigned char 	MSDresp[MAXSENDMSG];		// data returned by msd

struct MSwindow {
	float	ion;			// ion number
	float	dwell;			// dwell time (0 < dwell <= 1) (fraction of a second)
} MStable[MAXMSDCHANS][MAXMSDIONS];


int ReadMSDionFile ( char *IONFILE ) {
	FILE 	*fp;
	char 	buffer[1000];
	char 	*p;
	char	tmp[1];
	int		chan, inc;
	static char	once;
	double	ion, dwell;

	if (once == 1) return 0;

	fp = fopen(	IONFILE, "r" );
	if (fp == NULL) {
		fprintf(stderr, "Open failed: Cannont find %s\n", IONFILE);
		return -1;
	} else {
		while ( fgets( buffer, 1000, fp ) != NULL ) {

			// First strip out the MS channel number
			p = strstr( strlwr(buffer), "mschan" );
			strncpy(tmp, p + strlen("MSchan"), 1);
			chan = atoi(tmp);
			if (chan >= MAXMSDCHANS)
				break;
		
			// Next strip out the ions and dwell times
			p =	strtok(buffer, SEPCHARS );
			inc = 0;
			while ( p != NULL ) {

				if ( strstr(strlwr(p), "null") != NULL ) {
					MStable[chan][inc].ion = 0;
					MStable[chan][inc].dwell = 0;
					inc++;
				} else {
					ion = atof(p);
					if (ion >= 1) { 		// found ion
						p = strtok( NULL, SEPCHARS );
						dwell = atof(p);
						if (( dwell <= 1 ) && ( dwell > 0 )) {
							MStable[chan][inc].ion = ion;
							MStable[chan][inc].dwell = dwell;
							//printf ("%d,%d: %f %f\n", chan, inc, ion, dwell);
						} else {
							MStable[chan][inc].ion = 0;
							MStable[chan][inc].dwell = 0;
							fprintf(stderr, "Bad dwell time for channel %d, ion %f\n", chan, ion);
						}

						inc++;
					} else {
						MStable[chan][inc].ion = 0;
						MStable[chan][inc].dwell = 0;
						//printf("null ion: chan = %d, inc = %d\n", chan,inc);
					}

				}

				p = strtok( NULL, SEPCHARS );
			}
		}
	}

	fclose(fp);
	once = 1;

	return 0;

}

// Function takes a ion number and returns the MS channel number for
// a given time window.
int ion2chan( int win, float ion ) {

	int ch;
	float tol = 0.001;

	// Out of range variable
	if (win > MAXMSDIONS)
		return -1;

	// Scan MStable array for match
	for(ch=0; ch < MAXMSDCHANS; ch++) {
		if (((MStable[ch][win].ion - tol) < ion) && ((MStable[ch][win].ion + tol) > ion))
			return ch;
	}

	// No match in this time window
	return -1;

}

// Functions to pull data from MSD_Data and Time arrays.
long int MSD_Chan_Pull( int chan, int type ) {
	long msdDATA;
	static long msdTIMEold;

	if (currMSDPoint[chan] > MAXMSDPOINTS) 
		currMSDPoint[chan] = MAXMSDPOINTS;

	// type 0 is msd resp data, type 1 is msd time data.
	if ( type == 0 ) {
		msdDATA = MSD_Data_Array[chan][currMSDPoint[chan]];
		currMSDPoint[chan]++;
	} 
	else if ( type == 1 ) {		// real time displayed on text string.
		msdDATA = MSD_Time_Array[currMSDPoint[chan]-1];
		if ( msdDATA == MSDMISSING )
			msdDATA = msdTIMEold;
		else
			msdTIMEold = msdDATA;
	}
	else if ( type == 2 ) {		// this time needs to be monitonically increasing for rtg.
		msdDATA = MSD_Time_Array[currMSDPoint[chan]-1];
		if ( msdDATA == MSDMISSING )
			msdDATA = msdTIMEoldPlot++;
		else if ( msdDATA <= msdTIMEoldPlot ) {
			if ( msdDATA > msdTIMEoldPlot )
				msdDATA = msdDATA + 10;	
			else
				msdDATA = msdTIMEoldPlot + 10;
		}
		msdTIMEoldPlot = msdDATA;
	}

	return msdDATA;	
}




 typedef unsigned short WinNum;

 typedef long MSD_DataType;

MSD_DataType msdA_0, msdA_1, msdA_2, msdA_3, msdA_4, msdA_5, msdA_6, msdA_7, msdA_8;

int MSD_load( unsigned short IonWinNum ) {

	char *p;
	unsigned int chaninc[MAXMSDCHANS], inc, i, c;
	int chan = 0;
	float ion;

	// Initialize global currMSDPoint to 0
	for(c=0; c < MAXMSDCHANS; c++) {
		chaninc[c] = 0;
		currMSDPoint[c] = 0;
	}

	// Fills in MSD_Data_Array with "missing" data MSDMISSING
	for(i=0; i < MAXMSDPOINTS; i++) {
		for(c=0; c < MAXMSDCHANS; c++) {
			MSD_Data_Array[c][i] = MSDMISSING;
			if (c == 0)
				MSD_Time_Array[i] = MSDMISSING;
		}
	}

	// Parse data returned in string MSDresp
	p = strtok(MSDresp, ",");
	inc = 0;
	while( p != NULL ) {

		// Get ion number
		if ( (inc % 3) == 0 ) {
			ion = atof(p);
			chan = ion2chan(IonWinNum, ion);
			// printf("ion = %f, chan = %d\n", ion, chan);
		} 

		// response data
		else if ( (inc % 3) == 1 ) {
			// Set data array
			if ((chan >= 0) && (chan < MAXMSDCHANS)) {
				MSD_Data_Array[chan][chaninc[chan]] = atol(p);
				chaninc[chan]++;
				if (chaninc[chan] > MAXMSDPOINTS) {
					chaninc[chan] = MAXMSDPOINTS;
					break;
				}
			}
		} 

		// time data
		else if ( (inc % 3) == 2 ) {
			if (chan == 0)
				MSD_Time_Array[chaninc[chan] - 1] = atol(p);
		} 

		p = strtok(NULL, ",");
		inc++;
	}

	// Set MSDresp array to 0
	for (i=0; i<MAXSENDMSG; i++) 
		MSDresp[i] = '\0';
	
	return 1;

}		



	unsigned short InjectingInc_MSD = 0;


ULONG MSDchromtime;
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include "regnames.h"

#define MAXMON 	19		// should match number in msd driver

pid_t	MSDeng_proxy;
float 	MSDeng[MAXMON];

void InitializeMSDeng() {
	int param;
	static char once;

	if (once == 1) return;

	for (param=0; param<MAXMON; param++)
		MSDeng[param] = -99.0;

	once = 1;

}




 typedef double MSDdouble;
#include <sys/types.h>
#include "maycomm.h"
pid_t maycomm_proxy = 0;

TDLstruct tdl;

#define USELONGPATH  900	// use the long path TDL when less than 900 ppm



 typedef float tdlfloat;
 typedef float tdlfloas;
 typedef short tdlint6;
  #include "ttdriver.h"
  #define DG_CUSTOM_INIT tt_gc_reset()
  #define N_INJS_PER_FILE 1




UINT CurrInj_, CurrFile_, N_Inj_in_File_, Injected_;
UINT CurrInjMSD_, CurrFileMSD_, N_Inj_in_FileMSD_, InjectedMSD_;
  #include <ctype.h>
  static UINT read_gcnum( FILE *fp ) {
    int c;
	UINT v = 0;
	
    c = getc(fp);
	while ( isspace(c) ) c = getc(fp);
	while ( isdigit(c) ) {
	  v = v*10 + c - '0';
	  c = getc(fp);
	}
	return v;
  }

  static UINT read_msdnum( FILE *fp ) {
    int c;
	UINT v = 0;
	
    c = getc(fp);
	while ( isspace(c) ) c = getc(fp);
	while ( isdigit(c) ) {
	  v = v*10 + c - '0';
	  c = getc(fp);
	}
	return v;
  }

  void read_gcfile( UINT *injno, UINT *fileno ) {
	FILE *fp;
	
	fp = fopen( "GCFILE.CFG", "r" );
	if ( fp != 0 ) {
	  *injno = read_gcnum( fp );
	  *fileno = read_gcnum( fp );
	  fclose( fp );
	}
  }

  void read_msdfile( UINT *injno, UINT *fileno ) {
	FILE *fp;
	
	fp = fopen( "MSDFILE.CFG", "r" );
	if ( fp != 0 ) {
	  *injno = read_msdnum( fp );
	  *fileno = read_msdnum( fp );
	  fclose( fp );
	}
  }

  void write_gcfile( UINT injno, UINT fileno ) {
	FILE *fp;
	
	fp = fopen( "GCFILE.CFG", "w" );
	fprintf( fp, "%u\n%u\n\n\n", injno, fileno );
	fclose( fp );
  }

  void write_msdfile( UINT injno, UINT fileno ) {
	FILE *fp;
	
	fp = fopen( "MSDFILE.CFG", "w" );
	fprintf( fp, "%u\n%u\n\n\n", injno, fileno );
	fclose( fp );
  }


  static char nullregion[1];
  static unsigned char far *GCstatePtr = &nullregion;
  static unsigned char far *MSDstatePtr = &nullregion;

#ifdef OPT_COL_INIT
  #error OPT_COL_INIT is obsolete
#endif
#ifdef MSG_LABEL
  #error MSG_LABEL is obsolete
#endif

#ifdef N_CONSOLES
  #include "nl_cons.h"
  #ifndef DATA_ATTR
	#define DATA_ATTR 7
  #endif

  #define cdisplay(d,r,c,s) nlcon_display(d,r,c,s,DATA_ATTR)
  #define display_(d,r,c,t,v,s) {\
	static t sv_;\
	if (v != sv_) { cdisplay(d,r,c,s); sv_ = v; }\
  }
#endif

#ifdef CONSOLE_INIT
  #error CONSOLE_INIT is obsolete
#endif


unsigned char V0 = 0x1;
unsigned char V1 = 0x1;
unsigned char V2 = 0x1;
unsigned char V3 = 0x1;
unsigned char V4 = 0x1;
unsigned char V5 = 0x1;
unsigned char V6 = 0x1;
unsigned char V7 = 0x1;
unsigned char V8 = 0x1;
UINT CurrInj;
UINT CurrFile;
UCHAR GCstate;
UCHAR MSDstate;
UINT MFCtr, Synch;
InjStateT InjState;
adT_pump rpm_Ipump;
adT PresHead_P;
adT_flow FlowM_CO;
adT_flow FlowB_CO;
adT PresBP_CO;
adT PresE_CO;
vdc_chan VoltM_CO;
vdc_chan VoltB_CO;
vdc_chan VoltBP_CO;
vdc_chan VoltE_CO;
adT_flow FlowM_SF6;
adT_flow FlowB_SF6;
adT PresBP_SF6;
adT PresE_SF6;
vdc_chan VoltM_SF6;
vdc_chan VoltB_SF6;
vdc_chan VoltBP_SF6;
vdc_chan VoltE_SF6;
adT_flow FlowM_F11;
adT_flow FlowB_F11;
adT PresBP_F11;
adT PresE_F11;
vdc_chan VoltM_F11;
vdc_chan VoltB_F11;
vdc_chan VoltBP_F11;
vdc_chan VoltE_F11;
adT Pres1_PAN;
adT Pres2_PAN;
adT PresSrc_PAN;
adT_flow FlowSrc_PAN;
adT PresBP_PAN;
adT PresE_PAN;
vdc_chan VoltM_PAN;
vdc_chan VoltB_PAN;
vdc_chan VoltBP_PAN;
vdc_chan VoltE_PAN;
adT PresMS1;
adT PresMS2;
adT PresBP_5;
adT PresE_5;
vdc_chan VoltMS1;
vdc_chan VoltMS2;
vdc_chan VoltBP_5;
vdc_chan VoltE_5;
dacMSpr PresMS1sp;
dacMSpr PresMS2sp;
adT Trap1_set;
adT Trap1_re;
adT Trap2_set;
adT Trap2_re;
adT ColT_MS1;
adT ColT_MS2;
adT ColT_PAN;
adT Pres_SL;
adT Pres_Inlet;
adT Flow_MSsl;
adT Flow_Packsl;
adT Flow_PANsl;
adT PresH_CO2;
adT PresH_N2O;
adT PresH_ZA;
adT PresH_PAN;
adT PresH_Cal2;
adT PresH_Cal1;
adT PresH_He;
adT PresH_N2;
adT PresL_CO2;
adT PresL_N2O;
adT PresL_ZA;
adT PresL_PAN;
adT PresL_Cal2;
adT PresL_Cal1;
adT PresL_He;
adT PresL_N2;
adT Temp_F11sl;
adT Temp_SF6sl;
adT Temp_COsl;
adT Temp_PANsl;
adT Temp_GSVbo;
adT Temp_elect;
adT Temp_stack;
adT Temp_Ipump;
adT Temp_PANsrc;
adT Temp_GasBot;
adT Temp_PANlmp;
vdc_pow Pow_p28pri;
vdc_pow Pow_p28sec;
vdc_pow Pow_p15pel1;
vdc_pow Pow_p15pel2;
vdc_pow Pow_p12;
vdc_pow Pow_m15;
vdc_pow Pow_p15;
vdc_pow Pow_p5;
vdc_pow Pow_p24gsv;
vdc_pow Pow_p24ms;
Bit_HI_LO faildummy;
Bit_Cl_Op SLoopIN;
Bit_Cl_Op SLoopOUT;
Bit_Cl_Op PANflsh;
Bit_Op_Cl SLoopMSout;
Bit_HI_LO pilotTrip;
Bit_Off_On pelt1State;
Bit_Off_On pelt2State;
Bit_Off_On PANlamp;
Bit_Off_On PANflowCont;
Bit_Off_On trap1heat;
Bit_Off_On trap2heat;
Bit_Off_On trap1relay;
Bit_Off_On trap2relay;
Bit_Air_Cal PANsol1;
Bit_Cl_Op PANsol2;
Bit_Off_On PANsol3;
Bit_Op_Cl PANsol4;
OmegaT F11_Col;
OmegaT CO_Col;
OmegaT SF6_Col;
OmegaT SF6_Post;
OmegaT F11_ECD;
OmegaT CO_ECD;
OmegaT SF6_ECD;
OmegaT Trap1_CB;
OmegaT Trap2_CB;
OmegaT PAN_ECD;
OmegaTsp F11_Col_Sp;
OmegaTsp CO_Col_Sp;
OmegaTsp SF6_Col_Sp;
OmegaTsp SF6_Post_Sp;
OmegaTsp F11_ECD_Sp;
OmegaTsp CO_ECD_Sp;
OmegaTsp SF6_ECD_Sp;
OmegaTsp Trap1_CB_Sp;
OmegaTsp Trap2_CB_Sp;
OmegaTsp PAN_ECD_Sp;
GC_Ctr GC_Ctr_SF6;
GC_Ctr GC_Ctr_F11;
GC_Ctr GC_Ctr_CO;
GC_Ctr GC_Ctr_PAN;
valvT ssv1;
valvT ssv2;
valvetext valv_F11;
valvetext valv_CO;
valvetext valv_PAN;
valvetext valv_SF6;
valveMS1text valv_MS1;
valveMS2text valv_MS2;
WinNum IonWinNum;
UINT loadMSDarray;
MSD_DataType MSD_Chan0;
MSD_DataType MSD_Chan1;
MSD_DataType MSD_Chan2;
MSD_DataType MSD_Chan3;
MSD_DataType MSD_Chan4;
MSD_DataType MSD_Chan5;
MSD_DataType MSD_Time;
MSD_DataType MSD_TimePlot;
UINT CurrInj_MSD;
UINT CurrFile_MSD;
MSDdouble msdECUR;
MSDdouble msdIPOL;
MSDdouble msdREP;
MSDdouble msdEEGY;
MSDdouble msdAMUG;
MSDdouble msdAMUO;
MSDdouble msdIFOC;
MSDdouble msdENT;
MSDdouble msdFSEL;
MSDdouble msdMMW;
MSDdouble msdENTO;
MSDdouble msdHEDEN;
MSDdouble msdCAL;
MSDdouble msdEMUL;
MSDdouble msdSRCTEMP;
MSDdouble msdQUAD;
MSDdouble msdTURBOPCT;
MSDdouble msdMASG;
MSDdouble msdMASO;
ULONG maycX;
tdlfloas tdlP;
tdlfloas tdlT;
tdlfloat tdlsH2O;
tdlint6  tdlsAmp;
tdlint6  tdlsPow;
tdlint6  tdlsPos;
tdlint6  tdlsZer;
tdlfloat tdllH2O;
tdlint6  tdllAmp;
tdlint6  tdllPow;
tdlint6  tdllPos;
tdlint6  tdllZer;
tdlfloat tdlbH2O;
union {
  struct {
    UINT MFCtr;
    GC_Ctr GC_Ctr_CO;
    GC_Ctr GC_Ctr_F11;
    GC_Ctr GC_Ctr_PAN;
    GC_Ctr GC_Ctr_SF6;
    Bit_Off_On PANlamp;
    Bit_Off_On trap2heat;
    char U1[1];
    adT PresBP_CO;
    adT_flow FlowB_CO;
    adT_flow FlowM_CO;
    UINT Synch;
  } U0;
  struct {
    char U3[2];
    MSD_DataType MSD_TimePlot;
    MSD_DataType MSD_Time;
    MSD_DataType MSD_Chan5;
    MSD_DataType MSD_Chan4;
    Bit_Off_On trap1heat;
    Bit_Air_Cal PANsol1;
    char U4[2];
    UINT CurrFile;
    adT Pres2_PAN;
    char U5[1];
    adT_flow FlowM_SF6;
  } U2;
  struct {
    char U7[18];
    OmegaT SF6_Post;
    OmegaT F11_ECD;
    char U8[1];
    adT PresBP_F11;
    adT_flow FlowB_SF6;
    adT PresBP_SF6;
  } U6;
  struct {
    char U10[18];
    OmegaT Trap2_CB;
    OmegaT CO_ECD;
    char U11[1];
    adT PresBP_5;
    adT PresE_SF6;
    vdc_chan VoltM_SF6;
  } U9;
  struct {
    char U13[18];
    OmegaTsp F11_Col_Sp;
    OmegaTsp SF6_Post_Sp;
    char U14[2];
    vdc_chan VoltB_CO;
    UINT CurrInj;
    adT Pres1_PAN;
    UCHAR GCstate;
  } U12;
  struct {
    char U16[18];
    OmegaTsp CO_Col_Sp;
    OmegaTsp Trap2_CB_Sp;
    adT Temp_Ipump;
    adT Temp_PANsrc;
    adT Temp_SF6sl;
    UCHAR MSDstate;
  } U15;
  struct {
    char U18[18];
    valveMS2text valv_MS2;
    valveMS1text valv_MS1;
    Bit_HI_LO faildummy;
    vdc_chan VoltE_CO;
    vdc_chan VoltM_CO;
    char U19[3];
  } U17;
  struct {
    char U21[18];
    MSD_DataType MSD_Chan3;
    Bit_Cl_Op SLoopIN;
    vdc_chan VoltB_F11;
    char U22[2];
    Bit_Off_On pelt2State;
    char U23[3];
  } U20;
  struct {
    char U25[18];
    char U26[1];
    MSDdouble msdIPOL;
    vdc_chan VoltB_SF6;
    vdc_chan VoltBP_SF6;
  } U24;
  struct {
    char U28[18];
    char U29[2];
    MSDdouble msdEEGY;
    vdc_chan VoltM_F11;
    char U30[3];
  } U27;
  struct {
    char U32[18];
    char U33[3];
    MSDdouble msdAMUO;
    adT PresE_PAN;
    char U34[2];
  } U31;
  struct {
    char U36[18];
    char U37[4];
    Bit_Off_On trap2relay;
    char U38[2];
    Bit_Cl_Op SLoopOUT;
    vdc_chan VoltB_PAN;
    char U39[3];
  } U35;
  struct {
    char U41[18];
    char U42[5];
    MSDdouble msdMMW;
    adT_flow FlowM_F11;
  } U40;
  struct {
    char U44[18];
    char U45[6];
    char U46[1];
    adT_flow FlowSrc_PAN;
    char U47[6];
  } U43;
  struct {
    char U49[18];
    char U50[7];
    MSDdouble msdEMUL;
    char U51[2];
  } U48;
  struct {
    char U53[18];
    char U54[5];
    MSDdouble msdMASG;
    vdc_chan VoltBP_F11;
  } U52;
  struct {
    char U56[18];
    char U57[3];
    tdlfloat tdlsH2O;
    tdlfloas tdlT;
    tdlfloas tdlP;
    OmegaT CO_Col;
  } U55;
  struct {
    char U59[18];
    tdlint6  tdllZer;
    tdlint6  tdllPos;
    Bit_Cl_Op PANsol2;
    vdc_chan VoltE_PAN;
    vdc_chan VoltMS1;
    Bit_Cl_Op PANflsh;
    char U60[1];
  } U58;
  struct {
    char U62[22];
    OmegaTsp F11_ECD_Sp;
    vdc_chan VoltMS2;
    Bit_Off_On pelt1State;
    Bit_Off_On PANflowCont;
    Bit_HI_LO pilotTrip;
    char U63[1];
  } U61;
  struct {
    char U65[22];
    OmegaTsp CO_ECD_Sp;
    Bit_Op_Cl SLoopMSout;
    Bit_Op_Cl PANsol4;
    OmegaT Trap1_CB;
    OmegaT SF6_ECD;
    Bit_Off_On trap1relay;
    char U66[1];
  } U64;
  struct {
    char U68[22];
    valvetext valv_PAN;
    Bit_Off_On PANsol3;
    OmegaT SF6_Col;
    OmegaTsp SF6_Col_Sp;
    OmegaTsp Trap1_CB_Sp;
    OmegaT F11_Col;
    char U69[1];
  } U67;
  struct {
    char U71[22];
    MSD_DataType MSD_Chan2;
    OmegaT PAN_ECD;
    OmegaTsp PAN_ECD_Sp;
    valvT ssv2;
    OmegaTsp SF6_ECD_Sp;
    char U72[1];
  } U70;
  struct {
    char U74[22];
    MSDdouble msdENT;
    MSD_DataType MSD_Chan0;
    char U75[1];
  } U73;
  struct {
    char U77[22];
    char U78[4];
    valvetext valv_F11;
    valvetext valv_SF6;
    UINT CurrInj_MSD;
    valvT ssv1;
    char U79[1];
  } U76;
  struct {
    char U81[22];
    tdlint6  tdllPow;
    valvetext valv_CO;
    MSD_DataType MSD_Chan1;
    char U82[5];
  } U80;
  struct {
    char U84[24];
    WinNum IonWinNum;
    UINT CurrFile_MSD;
    char U85[7];
  } U83;
  struct {
    char U87[24];
    UINT loadMSDarray;
    MSDdouble msdQUAD;
    char U88[1];
  } U86;
  struct {
    char U90[24];
    MSDdouble msdHEDEN;
    char U91[3];
  } U89;
  struct {
    char U93[24];
    char U94[3];
    adT_flow FlowB_F11;
    adT PresMS1;
  } U92;
  struct {
    char U96[24];
    tdlint6  tdllAmp;
    ULONG maycX;
    tdlint6  tdlsPow;
    tdlint6  tdlsAmp;
    char U97[1];
  } U95;
  struct {
    char U99[26];
    tdlint6  tdlsZer;
    tdlint6  tdlsPos;
    tdlfloat tdlbH2O;
    char U100[1];
  } U98;
  struct {
    char U102[27];
    vdc_chan VoltE_F11;
    char U103[2];
    char U104[2];
  } U101;
  struct {
    char U106[27];
    adT PresMS2;
    char U107[4];
  } U105;
  struct {
    char U109[27];
    vdc_chan VoltE_5;
    char U110[4];
  } U108;
  struct {
    char U112[27];
    MSDdouble msdECUR;
  } U111;
  struct {
    char U114[34];
    char U115[1];
  } U113;
  struct {
    char U117[34];
    char U118[1];
  } U116;
  struct {
    char U120[34];
    char U121[1];
  } U119;
  struct {
    char U123[34];
    char U124[1];
  } U122;
  struct {
    char U126[34];
    char U127[1];
  } U125;
  struct {
    char U129[34];
    char U130[1];
  } U128;
  struct {
    char U132[34];
    char U133[1];
  } U131;
  struct {
    char U135[34];
    char U136[1];
  } U134;
  struct {
    char U138[34];
    char U139[1];
  } U137;
  struct {
    char U141[34];
    char U142[1];
  } U140;
  struct {
    char U144[34];
    char U145[1];
  } U143;
  struct {
    char U147[34];
    char U148[1];
  } U146;
  struct {
    char U150[34];
    char U151[1];
  } U149;
  struct {
    char U153[34];
    char U154[1];
  } U152;
  struct {
    char U156[34];
    char U157[1];
  } U155;
  struct {
    char U159[34];
    char U160[1];
  } U158;
  struct {
    char U162[34];
    char U163[1];
  } U161;
  struct {
    char U165[34];
    char U166[1];
  } U164;
  struct {
    char U168[34];
    char U169[1];
  } U167;
  struct {
    char U171[34];
    char U172[1];
  } U170;
  struct {
    char U174[34];
    char U175[1];
  } U173;
  struct {
    char U177[34];
    char U178[1];
  } U176;
  struct {
    char U180[34];
    char U181[1];
  } U179;
  struct {
    char U183[34];
    char U184[1];
  } U182;
  struct {
    char U186[34];
    char U187[1];
  } U185;
  struct {
    char U189[34];
    char U190[1];
  } U188;
  struct {
    char U192[34];
    char U193[1];
  } U191;
  struct {
    char U195[34];
    char U196[1];
  } U194;
  struct {
    char U198[34];
    char U199[1];
  } U197;
  struct {
    char U201[34];
    char U202[1];
  } U200;
  struct {
    char U204[34];
    char U205[1];
  } U203;
  struct {
    char U207[34];
    char U208[1];
  } U206;
  struct {
    char U210[34];
    char U211[1];
  } U209;
  struct {
    char U213[34];
    char U214[1];
  } U212;
  struct {
    char U216[34];
    char U217[1];
  } U215;
  struct {
    char U219[34];
    char U220[1];
  } U218;
  struct {
    char U222[34];
    char U223[1];
  } U221;
  struct {
    char U225[34];
    char U226[1];
  } U224;
  struct {
    char U228[34];
    char U229[1];
  } U227;
  struct {
    char U231[34];
    char U232[1];
  } U230;
  struct {
    char U234[34];
    char U235[1];
  } U233;
  struct {
    char U237[34];
    char U238[1];
  } U236;
  struct {
    char U240[34];
    char U241[1];
  } U239;
  struct {
    char U243[34];
    char U244[1];
  } U242;
  struct {
    char U246[34];
    char U247[1];
  } U245;
  struct {
    char U249[34];
    char U250[1];
  } U248;
  struct {
    char U252[34];
    char U253[1];
  } U251;
  struct {
    char U255[34];
    char U256[1];
  } U254;
  struct {
    char U258[34];
    char U259[1];
  } U257;
  struct {
    char U261[34];
    char U262[1];
  } U260;
  struct {
    char U264[34];
    char U265[1];
  } U263;
  struct {
    char U267[34];
    char U268[1];
  } U266;
  struct {
    char U270[34];
    char U271[1];
  } U269;
  struct {
    char U273[34];
    char U274[1];
  } U272;
  struct {
    char U276[34];
    char U277[1];
  } U275;
  struct {
    char U279[34];
    char U280[1];
  } U278;
  struct {
    char U282[34];
    char U283[1];
  } U281;
  struct {
    char U285[34];
    char U286[1];
  } U284;
  struct {
    char U288[34];
    char U289[1];
  } U287;
  struct {
    char U291[34];
    char U292[1];
  } U290;
  struct {
    char U294[34];
    char U295[1];
  } U293;
  struct {
    char U297[34];
    char U298[1];
  } U296;
  struct {
    char U300[34];
    char U301[1];
  } U299;
  struct {
    char U303[34];
    char U304[1];
  } U302;
  struct {
    char U306[34];
    char U307[1];
  } U305;
  struct {
    char U309[34];
    char U310[1];
  } U308;
  struct {
    char U312[34];
    char U313[1];
  } U311;
  struct {
    char U315[34];
    char U316[1];
  } U314;
  struct {
    char U318[34];
    char U319[1];
  } U317;
  struct {
    char U321[34];
    char U322[1];
  } U320;
  struct {
    char U324[34];
    char U325[1];
  } U323;
  struct {
    char U327[34];
    char U328[1];
  } U326;
  struct {
    char U330[34];
    char U331[1];
  } U329;
  struct {
    char U333[34];
    char U334[1];
  } U332;
  struct {
    char U336[34];
    char U337[1];
  } U335;
  struct {
    char U339[34];
    char U340[1];
  } U338;
  struct {
    char U342[34];
    char U343[1];
  } U341;
  struct {
    char U345[34];
    char U346[1];
  } U344;
  struct {
    char U348[34];
    char U349[1];
  } U347;
  struct {
    char U351[34];
    char U352[1];
  } U350;
  struct {
    char U354[34];
    char U355[1];
  } U353;
  struct {
    char U357[34];
    char U358[1];
  } U356;
  struct {
    char U360[34];
    char U361[1];
  } U359;
  struct {
    char U363[34];
    char U364[1];
  } U362;
  struct {
    char U366[34];
    char U367[1];
  } U365;
  struct {
    char U369[34];
    char U370[1];
  } U368;
  struct {
    char U372[34];
    char U373[1];
  } U371;
  struct {
    char U375[34];
    char U376[1];
  } U374;
  struct {
    char U378[34];
    char U379[1];
  } U377;
  struct {
    char U381[34];
    char U382[1];
  } U380;
  struct {
    char U384[34];
    char U385[1];
  } U383;
  struct {
    char U387[34];
    char U388[1];
  } U386;
  struct {
    char U390[34];
    char U391[1];
  } U389;
  struct {
    char U393[34];
    char U394[1];
  } U392;
  struct {
    char U396[34];
    char U397[1];
  } U395;
  struct {
    char U399[34];
    char U400[1];
  } U398;
  struct {
    char U402[34];
    char U403[1];
  } U401;
  struct {
    char U405[34];
    char U406[1];
  } U404;
  struct {
    char U408[34];
    char U409[1];
  } U407;
  struct {
    char U411[34];
    char U412[1];
  } U410;
  struct {
    char U414[34];
    char U415[1];
  } U413;
  struct {
    char U417[34];
    char U418[1];
  } U416;
  struct {
    char U420[34];
    char U421[1];
  } U419;
  struct {
    char U423[34];
    char U424[1];
  } U422;
  struct {
    char U426[34];
    char U427[1];
  } U425;
  struct {
    char U429[34];
    char U430[1];
  } U428;
  struct {
    char U432[34];
    char U433[1];
  } U431;
  struct {
    char U435[34];
    char U436[1];
  } U434;
  struct {
    char U438[34];
    char U439[1];
  } U437;
  struct {
    char U441[34];
    char U442[1];
  } U440;
  struct {
    char U444[34];
    char U445[1];
  } U443;
  struct {
    char U447[34];
    char U448[1];
  } U446;
  struct {
    char U450[34];
    char U451[1];
  } U449;
  struct {
    char U453[34];
    char U454[1];
  } U452;
  struct {
    char U456[34];
    char U457[1];
  } U455;
  struct {
    char U459[34];
    char U460[1];
  } U458;
  struct {
    char U462[34];
    char U463[1];
  } U461;
  struct {
    char U465[34];
    char U466[1];
  } U464;
  struct {
    char U468[34];
    char U469[1];
  } U467;
  struct {
    char U471[34];
    char U472[1];
  } U470;
  struct {
    char U474[34];
    char U475[1];
  } U473;
  struct {
    char U477[34];
    char U478[1];
  } U476;
  struct {
    char U480[34];
    char U481[1];
  } U479;
  struct {
    char U483[34];
    char U484[1];
  } U482;
  struct {
    char U486[34];
    char U487[1];
  } U485;
  struct {
    char U489[34];
    char U490[1];
  } U488;
  struct {
    char U492[34];
    char U493[1];
  } U491;
  struct {
    char U495[34];
    char U496[1];
  } U494;
  struct {
    char U498[34];
    char U499[1];
  } U497;
  struct {
    char U501[34];
    char U502[1];
  } U500;
  struct {
    char U504[34];
    char U505[1];
  } U503;
  struct {
    char U507[34];
    char U508[1];
  } U506;
  struct {
    char U510[34];
    char U511[1];
  } U509;
  struct {
    char U513[34];
    char U514[1];
  } U512;
  struct {
    char U516[34];
    char U517[1];
  } U515;
  struct {
    char U519[34];
    char U520[1];
  } U518;
  struct {
    char U522[34];
    char U523[1];
  } U521;
  struct {
    char U525[34];
    char U526[1];
  } U524;
  struct {
    char U528[34];
    char U529[1];
  } U527;
  struct {
    char U531[34];
    char U532[1];
  } U530;
  struct {
    char U534[34];
    char U535[1];
  } U533;
  struct {
    char U537[34];
    char U538[1];
  } U536;
  struct {
    char U540[34];
    char U541[1];
  } U539;
  struct {
    char U543[34];
    char U544[1];
  } U542;
  struct {
    char U546[34];
    char U547[1];
  } U545;
  struct {
    char U549[34];
    char U550[1];
  } U548;
  struct {
    char U552[34];
    char U553[1];
  } U551;
  struct {
    char U555[34];
    char U556[1];
  } U554;
  struct {
    char U558[34];
    char U559[1];
  } U557;
  struct {
    char U561[34];
    char U562[1];
  } U560;
  struct {
    char U564[34];
    char U565[1];
  } U563;
  struct {
    char U567[34];
    char U568[1];
  } U566;
  struct {
    char U570[34];
    char U571[1];
  } U569;
  struct {
    char U573[34];
    char U574[1];
  } U572;
  struct {
    char U576[34];
    char U577[1];
  } U575;
  struct {
    char U579[34];
    char U580[1];
  } U578;
  struct {
    char U582[34];
    char U583[1];
  } U581;
  struct {
    char U585[34];
    char U586[1];
  } U584;
  struct {
    char U588[34];
    char U589[1];
  } U587;
  struct {
    char U591[34];
    char U592[1];
  } U590;
  struct {
    char U594[34];
    char U595[1];
  } U593;
  struct {
    char U597[34];
    char U598[1];
  } U596;
  struct {
    char U600[34];
    char U601[1];
  } U599;
  struct {
    char U603[34];
    char U604[1];
  } U602;
  struct {
    char U606[34];
    char U607[1];
  } U605;
  struct {
    char U609[34];
    char U610[1];
  } U608;
  struct {
    char U612[34];
    char U613[1];
  } U611;
  struct {
    char U615[34];
    char U616[1];
  } U614;
  struct {
    char U618[34];
    char U619[1];
  } U617;
  struct {
    char U621[34];
    char U622[1];
  } U620;
  struct {
    char U624[34];
    char U625[1];
  } U623;
  struct {
    char U627[34];
    char U628[1];
  } U626;
  struct {
    char U630[34];
    char U631[1];
  } U629;
  struct {
    char U633[34];
    char U634[1];
  } U632;
  struct {
    char U636[34];
    char U637[1];
  } U635;
  struct {
    char U639[34];
    char U640[1];
  } U638;
  struct {
    char U642[34];
    char U643[1];
  } U641;
  struct {
    char U645[34];
    char U646[1];
  } U644;
  struct {
    char U648[34];
    char U649[1];
  } U647;
  struct {
    char U651[34];
    char U652[1];
  } U650;
  struct {
    char U654[34];
    char U655[1];
  } U653;
  struct {
    char U657[34];
    char U658[1];
  } U656;
  struct {
    char U660[34];
    char U661[1];
  } U659;
  struct {
    char U663[34];
    char U664[1];
  } U662;
  struct {
    char U666[34];
    char U667[1];
  } U665;
  struct {
    char U669[34];
    char U670[1];
  } U668;
  struct {
    char U672[34];
    char U673[1];
  } U671;
  struct {
    char U675[34];
    char U676[1];
  } U674;
  struct {
    char U678[34];
    char U679[1];
  } U677;
  struct {
    char U681[34];
    char U682[1];
  } U680;
  struct {
    char U684[34];
    char U685[1];
  } U683;
  struct {
    char U687[34];
    char U688[1];
  } U686;
  struct {
    char U690[34];
    char U691[1];
  } U689;
  struct {
    char U693[34];
    char U694[1];
  } U692;
  struct {
    char U696[34];
    char U697[1];
  } U695;
  struct {
    char U699[34];
    char U700[1];
  } U698;
  struct {
    char U702[34];
    char U703[1];
  } U701;
  struct {
    char U705[34];
    char U706[1];
  } U704;
  struct {
    char U708[34];
    char U709[1];
  } U707;
  struct {
    char U711[34];
    char U712[1];
  } U710;
  struct {
    char U714[34];
    char U715[1];
  } U713;
  struct {
    char U717[34];
    char U718[1];
  } U716;
  struct {
    char U720[34];
    char U721[1];
  } U719;
} *home;

static void VSET_3_(unsigned short newstate) {
  static unsigned short oldstate;
  
  switch (oldstate) {
    case 2: /* MaycommTog */
      V8 |= 0x1;
      break;
  }
  oldstate = newstate;
  switch (newstate) {
    case 2: /* MaycommTog */
      V8 &= ~0x1;
      break;
  }
}
static void VSET_2_(unsigned short newstate) {
  static unsigned short oldstate;
  
  switch (oldstate) {
    case 3: /* Injection_MSD */
      V6 |= 0x1;
      break;
    case 2: /* Injecting_MSD */ {
        V7 |= 0x1;
        V5 |= 0x1;
      }
      break;
  }
  oldstate = newstate;
  switch (newstate) {
    case 3: /* Injection_MSD */
      V6 &= ~0x1;
      break;
    case 2: /* Injecting_MSD */ {
        V7 &= ~0x1;
        V5 &= ~0x1;
        {
        	msdTIMEoldPlot = 0;
        } {
          if ( CurrFileMSD_ == 0 )
        	read_msdfile( &CurrInjMSD_, &CurrFileMSD_ );
          CurrInjMSD_++;
          if ( N_Inj_in_FileMSD_ == N_INJS_PER_FILE-1 ) {
            N_Inj_in_FileMSD_ = 0;
        	CurrFileMSD_++;
          } else N_Inj_in_FileMSD_++;
          write_msdfile( CurrInjMSD_, CurrFileMSD_ );
        }
      }
      break;
    case 1: /* NotInjecting_MSD */ {
        {
        	ReadMSDionFile( MSDIONFILE );
        } {
        	InitializeMSDeng();
        }
      }
      break;
  }
}
static void VSET_1_(unsigned short newstate) {
  static unsigned short oldstate;
  
  switch (oldstate) {
    case 2: /* GCTog */
      V3 |= 0x1;
      break;
  }
  oldstate = newstate;
  switch (newstate) {
    case 2: /* GCTog */
      V3 &= ~0x1;
      break;
  }
}
static void VSET_0_(unsigned short newstate) {
  static unsigned short oldstate;
  
  switch (oldstate) {
    case 3: /* Injection */
      V1 |= 0x1;
      break;
    case 2: /* Injecting */ {
        V4 |= 0x1;
        V2 |= 0x1;
        V0 |= 0x1;
      }
      break;
  }
  oldstate = newstate;
  switch (newstate) {
    case 3: /* Injection */
      V1 &= ~0x1;
      break;
    case 2: /* Injecting */ {
        V4 &= ~0x1;
        V2 &= ~0x1;
        V0 &= ~0x1;
        {
          if ( CurrFile_ == 0 )
        	read_gcfile( &CurrInj_, &CurrFile_ );
          CurrInj_++;
          if ( N_Inj_in_File_ == N_INJS_PER_FILE-1 ) {
            N_Inj_in_File_ = 0;
        	CurrFile_++;
          } else N_Inj_in_File_++;
          write_gcfile( CurrInj_, CurrFile_ );
        }
      }
      break;
  }
}
static void tminitfunc(void) {VSET_0_(1);VSET_1_(1);VSET_2_(1);VSET_3_(1);
}
#define TMINITFUNC tminitfunc()

static void nullfunc(void);
static void CF1_0(void) {
  home->U0.MFCtr = MFCtr;
  home->U0.Synch = Synch;
}

static void CF2_0(void) {
  CF1_0();
  {
  	MSD_Chan0 = MSD_Chan_Pull(0, 0);
  	MSD_Chan1 = MSD_Chan_Pull(1, 0);
  	MSD_Chan2 = MSD_Chan_Pull(2, 0);
  	MSD_Chan3 = MSD_Chan_Pull(3, 0);
  	MSD_Chan4 = MSD_Chan_Pull(4, 0);
  	MSD_Chan5 = MSD_Chan_Pull(5, 0);
  	MSD_Time  = MSD_Chan_Pull(0, 1);
  	MSD_TimePlot = MSD_Chan_Pull(0, 2);
  }
  home->U73.MSD_Chan0 = MSD_Chan0;
  home->U80.MSD_Chan1 = MSD_Chan1;
  home->U70.MSD_Chan2 = MSD_Chan2;
  home->U20.MSD_Chan3 = MSD_Chan3;
  home->U2.MSD_Chan4 = MSD_Chan4;
  home->U2.MSD_Chan5 = MSD_Chan5;
  home->U2.MSD_Time = MSD_Time;
  home->U2.MSD_TimePlot = MSD_TimePlot;
}

static void EF2_0(void) {
  {
  
  	for(chanInc=0; chanInc<MAXMSDCHANS; chanInc++) {
  		MSD_ChanX = MSDMISSING;
  		switch( chanInc ) {
  			case 0:
  				MSD_ChanX = MSD_Chan0; break;
  			case 1:
  				MSD_ChanX = MSD_Chan1; break;
  			case 2:
  				MSD_ChanX = MSD_Chan2; break;
  			case 3:
  				MSD_ChanX = MSD_Chan3; break;
  			case 4:
  				MSD_ChanX = MSD_Chan4; break;
  			case 5:
  				MSD_ChanX = MSD_Chan5; break;
  			case 6:
  				MSD_ChanX = MSD_Chan5; break;
  			case 7:
  				MSD_ChanX = MSD_Chan5; break;
  			default:
  				chanInc = MAXMSDCHANS-1; MSD_ChanX = MSDMISSING; break;
  		}
  
  		if ( MSD_ChanX > 0 ) {
  			msd_sum[chanInc] += MSD_ChanX;
  			MSchanAvg[chanInc]++;
  		}
  	}
  }
}

static void BF2_0(void) {
  CF2_0();
  EF2_0();
}

static void BF32_0(void) {
  CF2_0();
  GCstate = *GCstatePtr;
  home->U12.GCstate = GCstate;
  EF2_0();
  if (!(V0)) {
    {
      if ( GCstate == 2 ) VSET_0_(3);
    }
  }
    else {
      if ( GCstate == 1 ) VSET_0_(2);
      if ( GCstate == 0 ) VSET_0_(1);
    }
  {if (!(V1)) {
    { CHROMtime += 1; InjectingInc = 0; }
  }if (!(V2)) {
    { 
    	if (InjectingInc == 0 ) {
    		CHROMtime = 1; 
    		InjectingInc = 1;
    	}
    	else CHROMtime += 1;
      }
  }
  }
  {	
  	if (rs485_proxy != 0)  Trigger ( rs485_proxy );
  }
  {
  	if (ECDresp_proxy != 0) Trigger ( ECDresp_proxy );
  }
  {if (!(V3)) {
    { VSET_1_(1);
    	}
  }if (!(V4)) {
    V4 |= 0x1;
    { VSET_1_(2);
    		ecdSF6x = 0.0;
    		ecdF11x = 0.0;
    		ecdCOx  = 0.0;
    		ecdPANx = 0.0;
    	}
  }
  
  	if ( ecdSF6x > NUMSECSXAXIS ) { VSET_1_(2);
  		ecdSF6x = 0.0;
  		ecdF11x = 0.0;
  		ecdCOx  = 0.0;
  		ecdPANx = 0.0;
  	}
  }
  {
  	if ( DetAvgCt1 > 0 ) {
  		ecdA_SF6 = ecd_sum1/DetAvgCt1;
  		ecdA_SF6save = ecdA_SF6;
  		DetAvgCt1 = 0;
  		ecd_sum1 = 0;
  		DetAvgOffline1 = 0;
  	} 
  	else if (( ecdA_SF6save > 0 ) && ( DetAvgOffline1 < 1 )) {
  		DetAvgOffline1++;
  		ecdA_SF6 = ecdA_SF6save;
  	}
  	else {
  		ecdA_SF6 = ECDOFFLINE;
  		ecdA_SF6save = 0;
  	}
  
  	if ( DetAvgCt2 > 0 ) {
  		ecdA_F11 = ecd_sum2/DetAvgCt2;
  		ecdA_F11save = ecdA_F11;
  		DetAvgCt2 = 0;
  		ecd_sum2 = 0;
  		DetAvgOffline2 = 0;
  	} 
  	else if (( ecdA_F11save > 0 ) && ( DetAvgOffline2 < 1 )) {
  		DetAvgOffline2++;
  		ecdA_F11 = ecdA_F11save;
  	}
  	else {
  		ecdA_F11 = ECDOFFLINE;
  		ecdA_F11save = 0;
  	}
  
  	if ( DetAvgCt3 > 0 ) {
  		ecdA_CO = ecd_sum3/DetAvgCt3;
  		ecdA_COsave = ecdA_CO;
  		DetAvgCt3 = 0;
  		ecd_sum3 = 0;
  		DetAvgOffline3 = 0;
  	} 
  	else if (( ecdA_COsave > 0 ) && ( DetAvgOffline3 < 1 )) {
  		DetAvgOffline3++;
  		ecdA_CO = ecdA_COsave;
  	}
  	else {
  		ecdA_CO = ECDOFFLINE;
  		ecdA_COsave = 0;
  	}
  
  	if ( DetAvgCt4 > 0 ) {
  		ecdA_PAN = ecd_sum4/DetAvgCt4;
  		ecdA_PANsave = ecdA_PAN;
  		DetAvgCt4 = 0;
  		ecd_sum4 = 0;
  		DetAvgOffline4 = 0;
  	} 
  	else if (( ecdA_PANsave > 0 ) && ( DetAvgOffline4 < 1 )) {
  		DetAvgOffline4++;
  		ecdA_PAN = ecdA_PANsave;
  	}
  	else {
  		ecdA_PAN = ECDOFFLINE;
  		ecdA_PANsave = 0;
  	}
  
  }
  {
   	++CUMSEC;
  }
  {
  	if ( MSchanAvg[0] > 0 ) {
  		msdA_0 = msd_sum[0]/MSchanAvg[0];
  		MSchanAvg[0] = msd_sum[0] = 0;
  	} else {
  		msdA_0 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[1] > 0 ) {
  		msdA_1 = msd_sum[1]/MSchanAvg[1];
  		MSchanAvg[1] = msd_sum[1] = 0;
  	} else {
  		msdA_1 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[2] > 0 ) {
  		msdA_2 = msd_sum[2]/MSchanAvg[2];
  		MSchanAvg[2] = msd_sum[2] = 0;
  	} else {
  		msdA_2 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[3] > 0 ) {
  		msdA_3 = msd_sum[3]/MSchanAvg[3];
  		MSchanAvg[3] = msd_sum[3] = 0;
  	} else {
  		msdA_3 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[4] > 0 ) {
  		msdA_4 = msd_sum[4]/MSchanAvg[4];
  		MSchanAvg[4] = msd_sum[4] = 0;
  	} else {
  		msdA_4 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[5] > 0 ) {
  		msdA_5 = msd_sum[5]/MSchanAvg[5];
  		MSchanAvg[5] = msd_sum[5] = 0;
  	} else {
  		msdA_5 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[6] > 0 ) {
  		msdA_6 = msd_sum[6]/MSchanAvg[6];
  		MSchanAvg[6] = msd_sum[6] = 0;
  	} else {
  		msdA_6 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[7] > 0 ) {
  		msdA_7 = msd_sum[7]/MSchanAvg[7];
  		MSchanAvg[7] = msd_sum[7] = 0;
  	} else {
  		msdA_7 = MSDOFFLINE;
  	}
  
  }
  {if (!(V6)) {
    { 
    	MSDchromtime += 1; 
    	InjectingInc_MSD = 0;
      }
  }if (!(V7)) {
    { 
    	if ( InjectingInc_MSD == 0 ) {
    		MSDchromtime = 1; 
    		InjectingInc_MSD = 1;
    	} else 
    		MSDchromtime += 1;
      }
  }
  }
  {
  	if (maycomm_proxy != 0)  Trigger ( maycomm_proxy );
  }
}

static void CF2_1(void) {
  CF1_0();
  GC_Ctr_PAN = GC_Ctr_Pull(0x4);
  GC_Ctr_CO = GC_Ctr_Pull(0x3);
  GC_Ctr_F11 = GC_Ctr_Pull(0x2);
  GC_Ctr_SF6 = GC_Ctr_Pull(0x1);
  home->U0.GC_Ctr_SF6 = GC_Ctr_SF6;
  home->U0.GC_Ctr_PAN = GC_Ctr_PAN;
  home->U0.GC_Ctr_F11 = GC_Ctr_F11;
  home->U0.GC_Ctr_CO = GC_Ctr_CO;
}

static void EF2_1(void) {
  {
  	if (GC_Ctr_SF6 >= 0) {
  		ecd_sum1 += GC_Ctr_SF6;
  		DetAvgCt1++;
  		ecdSF6x += ELECRATE;
  	} 
  	if (GC_Ctr_F11 >= 0) {
  		ecd_sum2 += GC_Ctr_F11;
  		DetAvgCt2++;
  		ecdF11x += ELECRATE;
  	}
  	if (GC_Ctr_CO >= 0) {
  		ecd_sum3 += GC_Ctr_CO;
  		DetAvgCt3++;
  		ecdCOx += ELECRATE;
  	}
  	if (GC_Ctr_PAN >= 0) {
  		ecd_sum4 += GC_Ctr_PAN;
  		DetAvgCt4++;
  		ecdPANx += ELECRATE;
  	}
  }
}

static void CF8_1(void) {
  CF2_1();
  trap2heat = trapToggle(21);
  trap1heat = trapToggle(20);
  home->U0.trap2heat = trap2heat;
  home->U2.trap1heat = trap1heat;
}

static void CF16_1(void) {
  CF8_1();
  {
  	loadMSDarray = MSD_load( IonWinNum );
  	if ( MSDresp_proxy != 0 )
  		Trigger( MSDresp_proxy );
  }
  faildummy = fail_board(13);
  home->U86.loadMSDarray = loadMSDarray;
  home->U17.faildummy = faildummy;
}

static void BF32_1(void) {
  CF16_1();
  MSDstate = *MSDstatePtr;
  home->U15.MSDstate = MSDstate;
  home->U98.tdlbH2O = tdlbH2O;
  home->U95.maycX = maycX;
  EF2_1();
  {
  		MSDstateS = MSDstate;
  	}
  if (!(V5)) {
    {
      if ( MSDstate == 2 ) VSET_2_(3);
    }
  }
    else {
      if ( MSDstate == 1 ) VSET_2_(2);
      else if ( MSDstate == 0 ) VSET_2_(1);
    }
  maycX++;
  {if (!(V8)) {
    { VSET_3_(1); }
  }
  
  	if ( maycX >= 120 ) { 
  		maycX = 0; VSET_3_(2);
  	}
  }
}

static void BF320_2(void) {
  CF2_0();
  {
  		FlowM_SF6  = read_ad_value( tt_p4 +  6, flowH_R1, flowH_V1, flowH_V0 );
  		FlowB_SF6  = read_ad_value( tt_p4 +  8, flowH_R1, flowH_V1, flowH_V0 );
      	PresBP_SF6 = read_ad_value( tt_p4 + 10, presH_R1, presH_V1, presH_V0 );
  		PresE_SF6  = read_ad_value( tt_p3 + 10, presL_R1, presL_V1, presL_V0 );
  		VoltM_SF6  = read_atod( tt_p4 + 7 );
  		VoltB_SF6  = read_atod( tt_p4 + 9 );
      	VoltBP_SF6 = read_atod( tt_p4 + 11 );
      	VoltE_SF6  = read_atod( tt_p3 + 11 );
  	}
  memcpy(home->U113.U115, &VoltE_SF6, 1);
  EF2_0();
}

static void BF32_3(void) {
  CF2_1();
  {
  	tdlP = tdl.p;
  	tdlT = tdl.t;
  	tdlsH2O = tdl.s.h2o;
  	tdllH2O = tdl.l.h2o;
  	tdlsAmp = tdl.s.amp;
  	tdlsPow = tdl.s.pow;
  	tdlsPos = tdl.s.pos;
  	tdlsZer = tdl.s.zero;
  	tdllAmp = tdl.l.amp;
  	tdllPow = tdl.l.pow;
  	tdllPos = tdl.l.pos;
  	tdllZer = tdl.l.zero;
  }
  memcpy(home->U719.U721, &tdllH2O, 1);
  home->U95.tdlsAmp = tdlsAmp;
  home->U95.tdlsPow = tdlsPow;
  home->U98.tdlsPos = tdlsPos;
  home->U98.tdlsZer = tdlsZer;
  home->U95.tdllAmp = tdllAmp;
  home->U80.tdllPow = tdllPow;
  home->U58.tdllPos = tdllPos;
  home->U58.tdllZer = tdllZer;
  EF2_1();
}

static void BF320_4(void) {
  CF2_0();
  {
  		Flow_MSsl   = read_ad_value( tt_p1 + 6, flowPne_R1, flowPne_V1, flowPne_V0 );
  		Flow_Packsl = read_ad_value( tt_p1 + 5, flowH_R1, flowH_V1, flowH_V0 );
  		Flow_PANsl  = read_ad_value( tt_p1 + 4, flowH_R1, flowH_V1, flowH_V0 );
  	}
  memcpy(home->U305.U307, &Flow_PANsl, 1);
  EF2_0();
}

static void BF32_5(void) {
  CF2_1();
  CO_Col = OmegaArray( 2 * 0x2 - 2 );
  home->U55.tdlP = tdlP;
  home->U55.tdlT = tdlT;
  home->U55.tdlsH2O = tdlsH2O;
  memcpy(home->U55.U57, ((char*)(&tdllH2O))+1, 3);
  home->U55.CO_Col = CO_Col;
  EF2_1();
  {
  	tdlbH2O = tdllH2O;
  	if ( tdllH2O > USELONGPATH ) tdlbH2O = tdlsH2O; 
  }
}

static void BF320_6(void) {
  CF2_0();
  memcpy(home->U302.U304, ((char*)(&Flow_PANsl))+1, 1);
  EF2_0();
}

static void CF32_7(void) {
  CF2_1();
  {
  
  	ssv1 = valvePos[0];
  	ssv2 = valvePos[1];
  	valv_F11 = valvePos[2]-1;
  	valv_CO = valvePos[3]-1;
  	valv_PAN = valvePos[4]-1;
  	valv_SF6 = valvePos[5]-1;
  	valv_MS1 = valvePos[6]-1;
  	valv_MS2 = valvePos[7]-1;
  
  	if (valco_proxy != 0) 
  		Trigger ( valco_proxy );    
  
  }
  home->U76.ssv1 = ssv1;
  home->U70.ssv2 = ssv2;
  home->U76.valv_SF6 = valv_SF6;
  home->U76.valv_F11 = valv_F11;
  home->U80.valv_CO = valv_CO;
  home->U67.valv_PAN = valv_PAN;
  home->U17.valv_MS1 = valv_MS1;
  home->U17.valv_MS2 = valv_MS2;
}

static void BF320_7(void) {
  CF32_7();
  memcpy(home->U299.U301, ((char*)(&Flow_PANsl))+2, 1);
  EF2_1();
}

static void BF320_8(void) {
  CF2_0();
  memcpy(home->U296.U298, ((char*)(&Flow_PANsl))+3, 1);
  EF2_0();
}

static void CF32_9(void) {
  CF8_1();
  Temp_PANsrc = read_temp12v( tt_p2 + 8 );
  Temp_Ipump = read_temp( tt_p2 + 7 );
  Temp_SF6sl = read_temp( tt_p2 + 1 );
  home->U15.Temp_SF6sl = Temp_SF6sl;
  home->U15.Temp_PANsrc = Temp_PANsrc;
  home->U15.Temp_Ipump = Temp_Ipump;
}

static void BF320_9(void) {
  CF32_9();
  memcpy(home->U293.U295, &Flow_Packsl, 1);
  EF2_1();
}

static void BF320_10(void) {
  CF2_0();
  memcpy(home->U290.U292, ((char*)(&Flow_Packsl))+1, 1);
  EF2_0();
}

static void CF32_11(void) {
  CF2_1();
  CurrFile_MSD = CurrFileMSD_;
  CurrInj_MSD = CurrInjMSD_;
  CO_ECD_Sp = OmegaArray( 2 * 0x6 - 1 );
  CO_Col_Sp = OmegaArray( 2 * 0x2 - 1 );
  CO_ECD = OmegaArray( 2 * 0x6 - 2 );
  F11_Col = OmegaArray( 2 * 0x1 - 2 );
  CurrFile = CurrFile_;
  CurrInj = CurrInj_;
  home->U67.F11_Col = F11_Col;
  home->U76.CurrInj_MSD = CurrInj_MSD;
  home->U12.CurrInj = CurrInj;
  home->U83.CurrFile_MSD = CurrFile_MSD;
  home->U2.CurrFile = CurrFile;
  home->U64.CO_ECD_Sp = CO_ECD_Sp;
  home->U9.CO_ECD = CO_ECD;
  home->U15.CO_Col_Sp = CO_Col_Sp;
}

static void EF32_11(void) {
  EF2_1();
  { ITXFileNum = CurrInj * 1000 + CurrFile;
  }
}

static void BF320_11(void) {
  CF32_11();
  memcpy(home->U287.U289, ((char*)(&Flow_Packsl))+2, 1);
  EF32_11();
}

static void BF320_12(void) {
  CF2_0();
  memcpy(home->U284.U286, ((char*)(&Flow_Packsl))+3, 1);
  EF2_0();
}

static void CF32_13(void) {
  CF2_1();
  IonWinNum = cache_read(0x1000);
  PAN_ECD_Sp = OmegaArray( 2 * 0xA - 1 );
  F11_ECD_Sp = OmegaArray( 2 * 0x5 - 1 );
  F11_Col_Sp = OmegaArray( 2 * 0x1 - 1 );
  PAN_ECD = OmegaArray( 2 * 0xA - 2 );
  F11_ECD = OmegaArray( 2 * 0x5 - 2 );
  PANflowCont = read_dig_out(19);
  PANflsh = read_dig_out(1);
  home->U58.PANflsh = PANflsh;
  home->U61.PANflowCont = PANflowCont;
  home->U70.PAN_ECD_Sp = PAN_ECD_Sp;
  home->U70.PAN_ECD = PAN_ECD;
  home->U83.IonWinNum = IonWinNum;
  home->U61.F11_ECD_Sp = F11_ECD_Sp;
  home->U6.F11_ECD = F11_ECD;
  home->U12.F11_Col_Sp = F11_Col_Sp;
}

static void BF320_13(void) {
  CF32_13();
  memcpy(home->U281.U283, &Flow_MSsl, 1);
  EF2_1();
}

static void BF320_14(void) {
  CF2_0();
  memcpy(home->U278.U280, ((char*)(&Flow_MSsl))+1, 1);
  EF2_0();
}

static void CF32_15(void) {
  CF2_1();
  PANsol4 = read_dig_out(7);
  PANsol3 = read_dig_out(6);
  PANsol2 = read_dig_out(5);
  PANsol1 = read_dig_out(4);
  PANlamp = read_dig_out(18);
  pelt2State = read_dig_out(17);
  pelt1State = read_dig_out(16);
  pilotTrip = read_dig_in(24);
  home->U61.pilotTrip = pilotTrip;
  home->U20.pelt2State = pelt2State;
  home->U61.pelt1State = pelt1State;
  home->U64.PANsol4 = PANsol4;
  home->U67.PANsol3 = PANsol3;
  home->U58.PANsol2 = PANsol2;
  home->U2.PANsol1 = PANsol1;
  home->U0.PANlamp = PANlamp;
}

static void BF320_15(void) {
  CF32_15();
  memcpy(home->U275.U277, ((char*)(&Flow_MSsl))+2, 1);
  EF2_1();
}

static void BF320_16(void) {
  CF2_0();
  memcpy(home->U272.U274, ((char*)(&Flow_MSsl))+3, 1);
  EF2_0();
}

static void CF32_17(void) {
  CF16_1();
  SF6_ECD_Sp = OmegaArray( 2 * 0x7 - 1 );
  SF6_Col_Sp = OmegaArray( 2 * 0x3 - 1 );
  SF6_ECD = OmegaArray( 2 * 0x7 - 2 );
  SF6_Col = OmegaArray( 2 * 0x3 - 2 );
  home->U70.SF6_ECD_Sp = SF6_ECD_Sp;
  home->U64.SF6_ECD = SF6_ECD;
  home->U67.SF6_Col_Sp = SF6_Col_Sp;
  home->U67.SF6_Col = SF6_Col;
}

static void BF32_17(void) {
  CF32_17();
  EF2_1();
}

static void BF320_17(void) {
  CF32_17();
  ColT_MS1 = read_ad_value( tt_p1 + 0, tempTrap_R1, tempTrap_V1, tempTrap_V0 );
  memcpy(home->U221.U223, &ColT_MS1, 1);
  EF2_1();
}

static void BF320_18(void) {
  CF2_0();
  memcpy(home->U218.U220, ((char*)(&ColT_MS1))+1, 1);
  EF2_0();
}

static void CF32_19(void) {
  CF2_1();
  Trap1_CB_Sp = OmegaArray( 2 * 0x8 - 1 );
  SF6_Post_Sp = OmegaArray( 2 * 0x4 - 1 );
  Trap1_CB = OmegaArray( 2 * 0x8 - 2 );
  SF6_Post = OmegaArray( 2 * 0x4 - 2 );
  trap1relay = read_dig_out(22);
  SLoopMSout = read_dig_out(3);
  SLoopOUT = read_dig_out(2);
  SLoopIN = read_dig_out(0);
  home->U64.trap1relay = trap1relay;
  home->U67.Trap1_CB_Sp = Trap1_CB_Sp;
  home->U64.Trap1_CB = Trap1_CB;
  home->U35.SLoopOUT = SLoopOUT;
  home->U64.SLoopMSout = SLoopMSout;
  home->U20.SLoopIN = SLoopIN;
  home->U12.SF6_Post_Sp = SF6_Post_Sp;
  home->U6.SF6_Post = SF6_Post;
}

static void BF32_19(void) {
  CF32_19();
  EF2_1();
}

static void BF320_19(void) {
  CF32_19();
  memcpy(home->U215.U217, ((char*)(&ColT_MS1))+2, 1);
  EF2_1();
}

static void BF320_20(void) {
  CF2_0();
  memcpy(home->U212.U214, ((char*)(&ColT_MS1))+3, 1);
  EF2_0();
}

static void CF32_21(void) {
  CF2_1();
  Trap2_CB_Sp = OmegaArray( 2 * 0x9 - 1 );
  Trap2_CB = OmegaArray( 2 * 0x9 - 2 );
  trap2relay = read_dig_out(23);
  home->U35.trap2relay = trap2relay;
  home->U15.Trap2_CB_Sp = Trap2_CB_Sp;
  home->U9.Trap2_CB = Trap2_CB;
}

static void CF64_21(void) {
  CF32_21();
  {
  
  	msdECUR     = MSDeng[msg_MONecur];
  	msdIPOL     = MSDeng[msg_MONipol];
  	msdREP      = MSDeng[msg_MONrep];
  	msdEEGY     = MSDeng[msg_MONeegy];
  	msdAMUG     = MSDeng[msg_MONamug];
  	msdAMUO     = MSDeng[msg_MONamuo];
  	msdIFOC     = MSDeng[msg_MONifoc];
  	msdENT      = MSDeng[msg_MONent];
  	msdFSEL     = MSDeng[msg_MONfsel];
  	msdMMW      = MSDeng[msg_MONmmw];
  	msdENTO     = MSDeng[msg_MONento];
  	msdHEDEN    = MSDeng[msg_MONheden];
  	msdCAL      = MSDeng[msg_MONcal];
  	msdEMUL     = MSDeng[msg_MONemul];
  	msdSRCTEMP  = MSDeng[msg_MONsrctemp];
  	msdQUAD     = MSDeng[msg_MONquad];
  	msdTURBOPCT = MSDeng[msg_MONturbopct];
  	msdMASG     = MSDeng[msg_MONmasg];
  	msdMASO     = MSDeng[msg_MONmaso];
  
  	if ( MSDeng_proxy != 0 )
  		Trigger( MSDeng_proxy );
  
  }
  memcpy(home->U92.U94, &msdMASO, 3);
}

static void BF320_21(void) {
  CF64_21();
  {
  		PresMS1  = read_ad_value( tt_p3 + 0, presL30_R1, presL30_V1, presL30_V0 );
  		PresMS2  = read_ad_value( tt_p3 + 2, presL30_R1, presL30_V1, presL30_V0 );
      	PresBP_5 = read_ad_value( tt_p3 + 4, presH_R1, presH_V1, presH_V0 );
  		PresE_5  = read_ad_value( tt_p3 + 16, presL_R1, presL_V1, presL_V0 );
  		VoltMS1  = read_atod( tt_p3 + 1 );
  		VoltMS2  = read_atod( tt_p3 + 3 );
      	VoltBP_5 = read_atod( tt_p3 + 5 );
  	    VoltE_5  = read_atod( tt_p3 + 17 );
  	}
  memcpy(home->U31.U34, ((char*)(&VoltE_SF6))+1, 2);
  memcpy(home->U101.U103, &VoltBP_5, 2);
  home->U108.VoltE_5 = VoltE_5;
  EF2_1();
}

static void BF320_22(void) {
  CF2_0();
  ColT_MS2 = read_ad_value( tt_p1 + 1, tempTrap_R1, tempTrap_V1, tempTrap_V0 );
  memcpy(home->U233.U235, &ColT_MS2, 1);
  EF2_0();
}

static void BF64_23(void) {
  CF2_1();
  memcpy(home->U108.U110, &msdTURBOPCT, 4);
  home->U52.msdMASG = msdMASG;
  memcpy(home->U52.U54, ((char*)(&msdMASO))+3, 5);
  EF2_1();
}

static void BF320_24(void) {
  CF2_0();
  memcpy(home->U230.U232, ((char*)(&ColT_MS2))+1, 1);
  EF2_0();
}

static void BF64_25(void) {
  CF8_1();
  memcpy(home->U716.U718, &msdSRCTEMP, 1);
  home->U86.msdQUAD = msdQUAD;
  memcpy(home->U76.U78, ((char*)(&msdTURBOPCT))+4, 4);
  EF2_1();
}

static void BF320_26(void) {
  CF2_0();
  memcpy(home->U227.U229, ((char*)(&ColT_MS2))+2, 1);
  EF2_0();
}

static void BF64_27(void) {
  CF2_1();
  memcpy(home->U101.U104, &msdCAL, 2);
  home->U48.msdEMUL = msdEMUL;
  memcpy(home->U48.U50, ((char*)(&msdSRCTEMP))+1, 7);
  EF2_1();
}

static void BF320_28(void) {
  CF2_0();
  memcpy(home->U224.U226, ((char*)(&ColT_MS2))+3, 1);
  EF2_0();
}

static void BF64_29(void) {
  CF2_1();
  memcpy(home->U89.U91, &msdENTO, 3);
  home->U89.msdHEDEN = msdHEDEN;
  memcpy(home->U43.U45, ((char*)(&msdCAL))+2, 6);
  EF2_1();
}

static void BF320_30(void) {
  CF2_0();
  ColT_PAN = read_ad_value( tt_p1 + 3, tempTrap_R1, tempTrap_V1, tempTrap_V0 );
  memcpy(home->U245.U247, &ColT_PAN, 1);
  EF2_0();
}

static void BF64_31(void) {
  CF2_1();
  memcpy(home->U105.U107, &msdFSEL, 4);
  home->U40.msdMMW = msdMMW;
  memcpy(home->U40.U42, ((char*)(&msdENTO))+3, 5);
  EF2_1();
}

static void BF320_34(void) {
  CF2_0();
  memcpy(home->U242.U244, ((char*)(&ColT_PAN))+1, 1);
  EF2_0();
}

static void BF320_36(void) {
  CF2_0();
  memcpy(home->U239.U241, ((char*)(&ColT_PAN))+2, 1);
  EF2_0();
}

static void BF320_38(void) {
  CF2_0();
  memcpy(home->U236.U238, ((char*)(&ColT_PAN))+3, 1);
  EF2_0();
}

static void BF320_39(void) {
  CF32_7();
  Pow_m15 = read_atod( tt_p2 + 21 ) * 10;
  memcpy(home->U665.U667, &Pow_m15, 1);
  EF2_1();
}

static void BF320_40(void) {
  CF2_0();
  memcpy(home->U662.U664, ((char*)(&Pow_m15))+1, 1);
  EF2_0();
}

static void BF320_41(void) {
  CF32_9();
  memcpy(home->U659.U661, ((char*)(&Pow_m15))+2, 1);
  EF2_1();
}

static void BF320_42(void) {
  CF2_0();
  memcpy(home->U656.U658, ((char*)(&Pow_m15))+3, 1);
  EF2_0();
}

static void BF320_43(void) {
  CF32_11();
  Pow_p12 = read_atod( tt_p2 + 18 ) * 10;
  memcpy(home->U653.U655, &Pow_p12, 1);
  EF32_11();
}

static void BF320_44(void) {
  CF2_0();
  memcpy(home->U650.U652, ((char*)(&Pow_p12))+1, 1);
  EF2_0();
}

static void BF320_45(void) {
  CF32_13();
  memcpy(home->U647.U649, ((char*)(&Pow_p12))+2, 1);
  EF2_1();
}

static void BF320_46(void) {
  CF2_0();
  memcpy(home->U644.U646, ((char*)(&Pow_p12))+3, 1);
  EF2_0();
}

static void BF320_47(void) {
  CF32_15();
  Pow_p15 = read_atod( tt_p2 + 22 ) * 10;
  memcpy(home->U677.U679, &Pow_p15, 1);
  EF2_1();
}

static void BF320_48(void) {
  CF2_0();
  memcpy(home->U674.U676, ((char*)(&Pow_p15))+1, 1);
  EF2_0();
}

static void BF320_49(void) {
  CF32_17();
  memcpy(home->U671.U673, ((char*)(&Pow_p15))+2, 1);
  EF2_1();
}

static void BF320_50(void) {
  CF2_0();
  memcpy(home->U668.U670, ((char*)(&Pow_p15))+3, 1);
  EF2_0();
}

static void BF320_51(void) {
  CF32_19();
  Pow_p15pel1 = read_atod( tt_p2 + 14 ) * 10;
  memcpy(home->U629.U631, &Pow_p15pel1, 1);
  EF2_1();
}

static void BF320_52(void) {
  CF2_0();
  memcpy(home->U626.U628, ((char*)(&Pow_p15pel1))+1, 1);
  EF2_0();
}

static void BF320_53(void) {
  CF32_21();
  {
  		FlowM_CO  = read_ad_value( tt_p4 + 0, flowH_R1, flowH_V1, flowH_V0 );
  		FlowB_CO  = read_ad_value( tt_p4 + 2, flowH_R1, flowH_V1, flowH_V0 );
  	    PresBP_CO = read_ad_value( tt_p4 + 4, presH_R1, presH_V1, presH_V0 );
  		
  		PresE_CO  = read_ad_value( tt_p3 + 12, presL_R1, presL_V1, presL_V0 );
  		VoltM_CO  = read_atod( tt_p4 + 1 );
  		VoltB_CO  = read_atod( tt_p4 + 3 );
  	    VoltBP_CO = read_atod( tt_p4 + 5 );
      	VoltE_CO  = read_atod( tt_p3 + 13 );
  	}
  home->U24.VoltBP_SF6 = VoltBP_SF6;
  memcpy(home->U2.U5, ((char*)(&VoltE_SF6))+3, 1);
  memcpy(home->U20.U22, &VoltBP_CO, 2);
  home->U17.VoltE_CO = VoltE_CO;
  EF2_1();
}

static void BF320_54(void) {
  CF2_0();
  memcpy(home->U623.U625, ((char*)(&Pow_p15pel1))+2, 1);
  EF2_0();
}

static void BF64_55(void) {
  CF2_1();
  memcpy(home->U80.U82, &msdIFOC, 5);
  home->U73.msdENT = msdENT;
  memcpy(home->U35.U37, ((char*)(&msdFSEL))+4, 4);
  EF2_1();
}

static void BF320_56(void) {
  CF2_0();
  memcpy(home->U620.U622, ((char*)(&Pow_p15pel1))+3, 1);
  EF2_0();
}

static void BF320_57(void) {
  CF8_1();
  memcpy(home->U35.U39, &PresE_5, 3);
  home->U58.VoltMS1 = VoltMS1;
  home->U61.VoltMS2 = VoltMS2;
  memcpy(home->U12.U14, ((char*)(&VoltBP_5))+2, 2);
  EF2_1();
}

static void BF320_58(void) {
  CF2_0();
  Pow_p15pel2 = read_atod( tt_p2 + 15 ) * 10;
  memcpy(home->U641.U643, &Pow_p15pel2, 1);
  EF2_0();
}

static void BF64_59(void) {
  CF2_1();
  memcpy(home->U43.U47, &msdAMUG, 6);
  home->U31.msdAMUO = msdAMUO;
  memcpy(home->U31.U33, ((char*)(&msdIFOC))+5, 3);
  EF2_1();
}

static void BF320_60(void) {
  CF2_0();
  memcpy(home->U638.U640, ((char*)(&Pow_p15pel2))+1, 1);
  EF2_0();
}

static void BF64_61(void) {
  CF2_1();
  memcpy(home->U83.U85, &msdREP, 7);
  home->U27.msdEEGY = msdEEGY;
  memcpy(home->U27.U29, ((char*)(&msdAMUG))+6, 2);
  EF2_1();
}

static void BF320_62(void) {
  CF2_0();
  memcpy(home->U635.U637, ((char*)(&Pow_p15pel2))+2, 1);
  EF2_0();
}

static void BF64_63(void) {
  CF2_1();
  home->U111.msdECUR = msdECUR;
  home->U24.msdIPOL = msdIPOL;
  memcpy(home->U24.U26, ((char*)(&msdREP))+7, 1);
  EF2_1();
}

static void BF320_66(void) {
  CF2_0();
  memcpy(home->U632.U634, ((char*)(&Pow_p15pel2))+3, 1);
  EF2_0();
}

static void BF320_68(void) {
  CF2_0();
  Pow_p24gsv = read_atod( tt_p2 + 19 ) * 10;
  memcpy(home->U701.U703, &Pow_p24gsv, 1);
  EF2_0();
}

static void BF320_70(void) {
  CF2_0();
  memcpy(home->U698.U700, ((char*)(&Pow_p24gsv))+1, 1);
  EF2_0();
}

static void BF320_71(void) {
  CF32_7();
  memcpy(home->U695.U697, ((char*)(&Pow_p24gsv))+2, 1);
  EF2_1();
}

static void BF320_72(void) {
  CF2_0();
  memcpy(home->U692.U694, ((char*)(&Pow_p24gsv))+3, 1);
  EF2_0();
}

static void BF320_73(void) {
  CF32_9();
  Pow_p24ms = read_atod( tt_p2 + 20 ) * 10;
  memcpy(home->U713.U715, &Pow_p24ms, 1);
  EF2_1();
}

static void BF320_74(void) {
  CF2_0();
  memcpy(home->U710.U712, ((char*)(&Pow_p24ms))+1, 1);
  EF2_0();
}

static void BF320_75(void) {
  CF32_11();
  memcpy(home->U707.U709, ((char*)(&Pow_p24ms))+2, 1);
  EF32_11();
}

static void BF320_76(void) {
  CF2_0();
  memcpy(home->U704.U706, ((char*)(&Pow_p24ms))+3, 1);
  EF2_0();
}

static void BF320_77(void) {
  CF32_13();
  Pow_p28pri = read_atod( tt_p2 + 17 ) * 10;
  memcpy(home->U605.U607, &Pow_p28pri, 1);
  EF2_1();
}

static void BF320_78(void) {
  CF2_0();
  memcpy(home->U602.U604, ((char*)(&Pow_p28pri))+1, 1);
  EF2_0();
}

static void BF320_79(void) {
  CF32_15();
  memcpy(home->U599.U601, ((char*)(&Pow_p28pri))+2, 1);
  EF2_1();
}

static void BF320_80(void) {
  CF2_0();
  memcpy(home->U596.U598, ((char*)(&Pow_p28pri))+3, 1);
  EF2_0();
}

static void BF320_81(void) {
  CF32_17();
  Pow_p28sec = read_atod( tt_p2 + 13 ) * 10;
  memcpy(home->U617.U619, &Pow_p28sec, 1);
  EF2_1();
}

static void BF320_82(void) {
  CF2_0();
  memcpy(home->U614.U616, ((char*)(&Pow_p28sec))+1, 1);
  EF2_0();
}

static void BF320_83(void) {
  CF32_19();
  memcpy(home->U611.U613, ((char*)(&Pow_p28sec))+2, 1);
  EF2_1();
}

static void BF320_84(void) {
  CF2_0();
  memcpy(home->U608.U610, ((char*)(&Pow_p28sec))+3, 1);
  EF2_0();
}

static void BF320_85(void) {
  CF64_21();
  {
  		FlowM_F11  = read_ad_value( tt_p4 + 12, flowH_R1, flowH_V1, flowH_V0 );
  		FlowB_F11  = read_ad_value( tt_p4 + 14, flowH_R1, flowH_V1, flowH_V0 );
  	    PresBP_F11 = read_ad_value( tt_p4 + 16, presH_R1, presH_V1, presH_V0 );
  		
  		PresE_F11  = read_ad_value( tt_p3 + 12, presL_R1, presL_V1, presL_V0 );
  		VoltM_F11  = read_atod( tt_p4 + 13 );
  		VoltB_F11  = read_atod( tt_p4 + 15 );
  	    VoltBP_F11 = read_atod( tt_p4 + 17 );
      	VoltE_F11  = read_atod( tt_p3 + 13 );
  	}
  home->U52.VoltBP_F11 = VoltBP_F11;
  home->U101.VoltE_F11 = VoltE_F11;
  EF2_1();
}

static void BF320_86(void) {
  CF2_0();
  Pow_p5 = read_atod( tt_p2 + 23 );
  memcpy(home->U689.U691, &Pow_p5, 1);
  EF2_0();
}

static void BF320_88(void) {
  CF2_0();
  memcpy(home->U686.U688, ((char*)(&Pow_p5))+1, 1);
  EF2_0();
}

static void BF320_90(void) {
  CF2_0();
  memcpy(home->U683.U685, ((char*)(&Pow_p5))+2, 1);
  EF2_0();
}

static void BF320_92(void) {
  CF2_0();
  memcpy(home->U680.U682, ((char*)(&Pow_p5))+3, 1);
  EF2_0();
}

static void BF320_94(void) {
  CF2_0();
  Pres_Inlet = read_ad_value(tt_p3 + 6, presL_R1, presL_V1, presL_V0 );
  memcpy(home->U269.U271, &Pres_Inlet, 1);
  EF2_0();
}

static void BF320_98(void) {
  CF2_0();
  memcpy(home->U266.U268, ((char*)(&Pres_Inlet))+1, 1);
  EF2_0();
}

static void BF320_100(void) {
  CF2_0();
  memcpy(home->U263.U265, ((char*)(&Pres_Inlet))+2, 1);
  EF2_0();
}

static void BF320_102(void) {
  CF2_0();
  memcpy(home->U260.U262, ((char*)(&Pres_Inlet))+3, 1);
  EF2_0();
}

static void BF320_103(void) {
  CF32_7();
  Pres_SL    = read_ad_value(tt_p3 + 8, presL_R1, presL_V1, presL_V0 );
  memcpy(home->U257.U259, &Pres_SL, 1);
  EF2_1();
}

static void BF320_104(void) {
  CF2_0();
  memcpy(home->U254.U256, ((char*)(&Pres_SL))+1, 1);
  EF2_0();
}

static void BF320_105(void) {
  CF32_9();
  memcpy(home->U251.U253, ((char*)(&Pres_SL))+2, 1);
  EF2_1();
}

static void BF320_106(void) {
  CF2_0();
  memcpy(home->U248.U250, ((char*)(&Pres_SL))+3, 1);
  EF2_0();
}

static void BF320_107(void) {
  CF32_11();
  memcpy(home->U125.U127, &PresBP_PAN, 1);
  EF32_11();
}

static void BF320_108(void) {
  CF2_0();
  memcpy(home->U122.U124, ((char*)(&PresBP_PAN))+1, 1);
  EF2_0();
}

static void BF320_109(void) {
  CF32_13();
  memcpy(home->U119.U121, ((char*)(&PresBP_PAN))+2, 1);
  EF2_1();
}

static void BF320_110(void) {
  CF2_0();
  memcpy(home->U116.U118, ((char*)(&PresBP_PAN))+3, 1);
  EF2_0();
}

static void BF320_111(void) {
  CF32_15();
  PresH_Cal1 = read_ad_value( tt_p1 + 19, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  memcpy(home->U377.U379, &PresH_Cal1, 1);
  EF2_1();
}

static void BF320_112(void) {
  CF2_0();
  memcpy(home->U374.U376, ((char*)(&PresH_Cal1))+1, 1);
  EF2_0();
}

static void BF320_113(void) {
  CF32_17();
  memcpy(home->U371.U373, ((char*)(&PresH_Cal1))+2, 1);
  EF2_1();
}

static void BF320_114(void) {
  CF2_0();
  memcpy(home->U368.U370, ((char*)(&PresH_Cal1))+3, 1);
  EF2_0();
}

static void BF320_115(void) {
  CF32_19();
  PresH_Cal2 = read_ad_value( tt_p1 + 17, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  memcpy(home->U365.U367, &PresH_Cal2, 1);
  EF2_1();
}

static void BF320_116(void) {
  CF2_0();
  memcpy(home->U362.U364, ((char*)(&PresH_Cal2))+1, 1);
  EF2_0();
}

static void BF320_117(void) {
  CF32_21();
  memcpy(home->U20.U23, &PresE_F11, 3);
  home->U27.VoltM_F11 = VoltM_F11;
  home->U20.VoltB_F11 = VoltB_F11;
  EF2_1();
}

static void BF320_118(void) {
  CF2_0();
  memcpy(home->U359.U361, ((char*)(&PresH_Cal2))+2, 1);
  EF2_0();
}

static void BF320_120(void) {
  CF2_0();
  memcpy(home->U356.U358, ((char*)(&PresH_Cal2))+3, 1);
  EF2_0();
}

static void BF320_121(void) {
  CF8_1();
  home->U92.PresMS1 = PresMS1;
  home->U105.PresMS2 = PresMS2;
  home->U9.PresBP_5 = PresBP_5;
  memcpy(home->U9.U11, ((char*)(&PresE_5))+3, 1);
  EF2_1();
}

static void BF320_122(void) {
  CF2_0();
  PresH_CO2  = read_ad_value( tt_p1 +  9, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  memcpy(home->U317.U319, &PresH_CO2, 1);
  EF2_0();
}

static void BF320_124(void) {
  CF2_0();
  memcpy(home->U314.U316, ((char*)(&PresH_CO2))+1, 1);
  EF2_0();
}

static void BF320_126(void) {
  CF2_0();
  memcpy(home->U311.U313, ((char*)(&PresH_CO2))+2, 1);
  EF2_0();
}

static void BF320_130(void) {
  CF2_0();
  memcpy(home->U308.U310, ((char*)(&PresH_CO2))+3, 1);
  EF2_0();
}

static void BF320_132(void) {
  CF2_0();
  PresH_He   = read_ad_value( tt_p1 + 21, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  memcpy(home->U389.U391, &PresH_He, 1);
  EF2_0();
}

static void BF320_134(void) {
  CF2_0();
  memcpy(home->U386.U388, ((char*)(&PresH_He))+1, 1);
  EF2_0();
}

static void BF320_135(void) {
  CF32_7();
  memcpy(home->U383.U385, ((char*)(&PresH_He))+2, 1);
  EF2_1();
}

static void BF320_136(void) {
  CF2_0();
  memcpy(home->U380.U382, ((char*)(&PresH_He))+3, 1);
  EF2_0();
}

static void BF320_137(void) {
  CF32_9();
  PresH_N2   = read_ad_value( tt_p1 + 23, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  memcpy(home->U401.U403, &PresH_N2, 1);
  EF2_1();
}

static void BF320_138(void) {
  CF2_0();
  memcpy(home->U398.U400, ((char*)(&PresH_N2))+1, 1);
  EF2_0();
}

static void BF320_139(void) {
  CF32_11();
  memcpy(home->U395.U397, ((char*)(&PresH_N2))+2, 1);
  EF32_11();
}

static void BF320_140(void) {
  CF2_0();
  memcpy(home->U392.U394, ((char*)(&PresH_N2))+3, 1);
  EF2_0();
}

static void BF320_141(void) {
  CF32_13();
  PresH_N2O  = read_ad_value( tt_p1 + 11, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  memcpy(home->U329.U331, &PresH_N2O, 1);
  EF2_1();
}

static void BF320_142(void) {
  CF2_0();
  memcpy(home->U326.U328, ((char*)(&PresH_N2O))+1, 1);
  EF2_0();
}

static void BF320_143(void) {
  CF32_15();
  memcpy(home->U323.U325, ((char*)(&PresH_N2O))+2, 1);
  EF2_1();
}

static void BF320_144(void) {
  CF2_0();
  memcpy(home->U320.U322, ((char*)(&PresH_N2O))+3, 1);
  EF2_0();
}

static void BF320_145(void) {
  CF32_17();
  PresH_PAN  = read_ad_value( tt_p1 + 15, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  memcpy(home->U353.U355, &PresH_PAN, 1);
  EF2_1();
}

static void BF320_146(void) {
  CF2_0();
  memcpy(home->U350.U352, ((char*)(&PresH_PAN))+1, 1);
  EF2_0();
}

static void BF320_147(void) {
  CF32_19();
  memcpy(home->U347.U349, ((char*)(&PresH_PAN))+2, 1);
  EF2_1();
}

static void BF320_148(void) {
  CF2_0();
  memcpy(home->U344.U346, ((char*)(&PresH_PAN))+3, 1);
  EF2_0();
}

static void BF320_149(void) {
  CF64_21();
  home->U9.VoltM_SF6 = VoltM_SF6;
  home->U24.VoltB_SF6 = VoltB_SF6;
  EF2_1();
}

static void BF320_150(void) {
  CF2_0();
  PresH_ZA   = read_ad_value( tt_p1 + 13, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  memcpy(home->U341.U343, &PresH_ZA, 1);
  EF2_0();
}

static void BF320_152(void) {
  CF2_0();
  memcpy(home->U338.U340, ((char*)(&PresH_ZA))+1, 1);
  EF2_0();
}

static void BF320_154(void) {
  CF2_0();
  memcpy(home->U335.U337, ((char*)(&PresH_ZA))+2, 1);
  EF2_0();
}

static void BF320_156(void) {
  CF2_0();
  memcpy(home->U332.U334, ((char*)(&PresH_ZA))+3, 1);
  EF2_0();
}

static void BF320_158(void) {
  CF2_0();
  PresHead_P = read_ad_value(tt_p1 + 2, presL_R1, presL_V1, presL_V0 );
  memcpy(home->U98.U100, &PresHead_P, 1);
  EF2_0();
}

static void BF320_162(void) {
  CF2_0();
  memcpy(home->U95.U97, ((char*)(&PresHead_P))+1, 1);
  EF2_0();
}

static void BF320_164(void) {
  CF2_0();
  memcpy(home->U86.U88, ((char*)(&PresHead_P))+2, 1);
  EF2_0();
}

static void BF320_166(void) {
  CF2_0();
  memcpy(home->U76.U79, ((char*)(&PresHead_P))+3, 1);
  EF2_0();
}

static void BF320_167(void) {
  CF32_7();
  PresL_Cal1 = read_ad_value( tt_p1 + 18, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  memcpy(home->U473.U475, &PresL_Cal1, 1);
  EF2_1();
}

static void BF320_168(void) {
  CF2_0();
  memcpy(home->U470.U472, ((char*)(&PresL_Cal1))+1, 1);
  EF2_0();
}

static void BF320_169(void) {
  CF32_9();
  memcpy(home->U467.U469, ((char*)(&PresL_Cal1))+2, 1);
  EF2_1();
}

static void BF320_170(void) {
  CF2_0();
  memcpy(home->U464.U466, ((char*)(&PresL_Cal1))+3, 1);
  EF2_0();
}

static void BF320_171(void) {
  CF32_11();
  PresL_Cal2 = read_ad_value( tt_p1 + 16, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  memcpy(home->U461.U463, &PresL_Cal2, 1);
  EF32_11();
}

static void BF320_172(void) {
  CF2_0();
  memcpy(home->U458.U460, ((char*)(&PresL_Cal2))+1, 1);
  EF2_0();
}

static void BF320_173(void) {
  CF32_13();
  memcpy(home->U455.U457, ((char*)(&PresL_Cal2))+2, 1);
  EF2_1();
}

static void BF320_174(void) {
  CF2_0();
  memcpy(home->U452.U454, ((char*)(&PresL_Cal2))+3, 1);
  EF2_0();
}

static void BF320_175(void) {
  CF32_15();
  PresL_CO2  = read_ad_value( tt_p1 +  8, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  memcpy(home->U413.U415, &PresL_CO2, 1);
  EF2_1();
}

static void BF320_176(void) {
  CF2_0();
  memcpy(home->U410.U412, ((char*)(&PresL_CO2))+1, 1);
  EF2_0();
}

static void BF320_177(void) {
  CF32_17();
  memcpy(home->U407.U409, ((char*)(&PresL_CO2))+2, 1);
  EF2_1();
}

static void BF320_178(void) {
  CF2_0();
  memcpy(home->U404.U406, ((char*)(&PresL_CO2))+3, 1);
  EF2_0();
}

static void BF320_179(void) {
  CF32_19();
  PresL_He   = read_ad_value( tt_p1 + 20, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  memcpy(home->U485.U487, &PresL_He, 1);
  EF2_1();
}

static void BF320_180(void) {
  CF2_0();
  memcpy(home->U482.U484, ((char*)(&PresL_He))+1, 1);
  EF2_0();
}

static void BF320_181(void) {
  CF32_21();
  {
  		Pres1_PAN  = read_ad_value( tt_p4 + 18, presL30_R1, presL30_V1, presL30_V0 );
  		Pres2_PAN  = read_ad_value( tt_p4 + 20, presL30_R1, presL30_V1, presL30_V0 );
  		PresSrc_PAN = read_ad_value( tt_p2 + 16, presPne30_R1, presPne30_V1, presPne30_V0 );
  		PresE_PAN  = read_ad_value( tt_p3 + 14, presL_R1, presL_V1, presL_V0 );
  		FlowSrc_PAN = read_ad_value( tt_p2 + 12, flowPne_R1, flowPne_V1, flowPne_V0 );
  		VoltM_PAN  = read_atod( tt_p4 + 19 );
  		VoltB_PAN  = read_atod( tt_p4 + 21 );
  	    VoltE_PAN  = read_atod( tt_p3 + 15 );
      	PresBP_PAN = read_ad_value( tt_p4 + 22, presL30_R1, presL30_V1, presL30_V0 );
      	VoltBP_PAN = read_atod( tt_p4 + 23 );
  	}
  memcpy(home->U27.U30, &VoltM_PAN, 3);
  home->U35.VoltB_PAN = VoltB_PAN;
  home->U58.VoltE_PAN = VoltE_PAN;
  EF2_1();
}

static void BF320_182(void) {
  CF2_0();
  memcpy(home->U479.U481, ((char*)(&PresL_He))+2, 1);
  EF2_0();
}

static void BF320_184(void) {
  CF2_0();
  memcpy(home->U476.U478, ((char*)(&PresL_He))+3, 1);
  EF2_0();
}

static void BF320_185(void) {
  CF8_1();
  memcpy(home->U17.U19, &PresE_CO, 3);
  home->U17.VoltM_CO = VoltM_CO;
  home->U12.VoltB_CO = VoltB_CO;
  memcpy(home->U2.U4, ((char*)(&VoltBP_CO))+2, 2);
  EF2_1();
}

static void BF320_186(void) {
  CF2_0();
  PresL_N2   = read_ad_value( tt_p1 + 22, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  memcpy(home->U497.U499, &PresL_N2, 1);
  EF2_0();
}

static void BF320_188(void) {
  CF2_0();
  memcpy(home->U494.U496, ((char*)(&PresL_N2))+1, 1);
  EF2_0();
}

static void BF320_190(void) {
  CF2_0();
  memcpy(home->U491.U493, ((char*)(&PresL_N2))+2, 1);
  EF2_0();
}

static void BF320_194(void) {
  CF2_0();
  memcpy(home->U488.U490, ((char*)(&PresL_N2))+3, 1);
  EF2_0();
}

static void BF320_196(void) {
  CF2_0();
  PresL_N2O  = read_ad_value( tt_p1 + 10, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  memcpy(home->U425.U427, &PresL_N2O, 1);
  EF2_0();
}

static void BF320_198(void) {
  CF2_0();
  memcpy(home->U422.U424, ((char*)(&PresL_N2O))+1, 1);
  EF2_0();
}

static void BF320_199(void) {
  CF32_7();
  memcpy(home->U419.U421, ((char*)(&PresL_N2O))+2, 1);
  EF2_1();
}

static void BF320_200(void) {
  CF2_0();
  memcpy(home->U416.U418, ((char*)(&PresL_N2O))+3, 1);
  EF2_0();
}

static void BF320_201(void) {
  CF32_9();
  PresL_PAN  = read_ad_value( tt_p1 + 14, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  memcpy(home->U449.U451, &PresL_PAN, 1);
  EF2_1();
}

static void BF320_202(void) {
  CF2_0();
  memcpy(home->U446.U448, ((char*)(&PresL_PAN))+1, 1);
  EF2_0();
}

static void BF320_203(void) {
  CF32_11();
  memcpy(home->U443.U445, ((char*)(&PresL_PAN))+2, 1);
  EF32_11();
}

static void BF320_204(void) {
  CF2_0();
  memcpy(home->U440.U442, ((char*)(&PresL_PAN))+3, 1);
  EF2_0();
}

static void BF320_205(void) {
  CF32_13();
  PresL_ZA   = read_ad_value( tt_p1 + 12, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  memcpy(home->U437.U439, &PresL_ZA, 1);
  EF2_1();
}

static void BF320_206(void) {
  CF2_0();
  memcpy(home->U434.U436, ((char*)(&PresL_ZA))+1, 1);
  EF2_0();
}

static void BF320_207(void) {
  CF32_15();
  memcpy(home->U431.U433, ((char*)(&PresL_ZA))+2, 1);
  EF2_1();
}

static void BF320_208(void) {
  CF2_0();
  memcpy(home->U428.U430, ((char*)(&PresL_ZA))+3, 1);
  EF2_0();
}

static void BF320_209(void) {
  CF32_17();
  PresMS1sp = convert_VtoR( 10.0 * tt_dac_in(0xC) / 4095.0, presL30_R1, presL30_V1, presL30_V0 );
  memcpy(home->U149.U151, &PresMS1sp, 1);
  EF2_1();
}

static void BF320_210(void) {
  CF2_0();
  memcpy(home->U146.U148, ((char*)(&PresMS1sp))+1, 1);
  EF2_0();
}

static void BF320_211(void) {
  CF32_19();
  memcpy(home->U143.U145, ((char*)(&PresMS1sp))+2, 1);
  EF2_1();
}

static void BF320_212(void) {
  CF2_0();
  memcpy(home->U140.U142, ((char*)(&PresMS1sp))+3, 1);
  EF2_0();
}

static void BF320_213(void) {
  CF64_21();
  home->U6.PresBP_SF6 = PresBP_SF6;
  home->U9.PresE_SF6 = PresE_SF6;
  EF2_1();
}

static void BF320_214(void) {
  CF2_0();
  PresMS2sp = convert_VtoR( 10.0 * tt_dac_in(0xD) / 4095.0, presL30_R1, presL30_V1, presL30_V0 );
  memcpy(home->U161.U163, &PresMS2sp, 1);
  EF2_0();
}

static void BF320_216(void) {
  CF2_0();
  memcpy(home->U158.U160, ((char*)(&PresMS2sp))+1, 1);
  EF2_0();
}

static void BF320_218(void) {
  CF2_0();
  memcpy(home->U155.U157, ((char*)(&PresMS2sp))+2, 1);
  EF2_0();
}

static void BF320_220(void) {
  CF2_0();
  memcpy(home->U152.U154, ((char*)(&PresMS2sp))+3, 1);
  EF2_0();
}

static void BF320_222(void) {
  CF2_0();
  rpm_Ipump = read_ad_value(tt_p1+7, 7000, 11, 0);
  memcpy(home->U73.U75, &rpm_Ipump, 1);
  EF2_0();
}

static void BF320_226(void) {
  CF2_0();
  memcpy(home->U70.U72, ((char*)(&rpm_Ipump))+1, 1);
  EF2_0();
}

static void BF320_228(void) {
  CF2_0();
  memcpy(home->U67.U69, ((char*)(&rpm_Ipump))+2, 1);
  EF2_0();
}

static void BF320_230(void) {
  CF2_0();
  memcpy(home->U64.U66, ((char*)(&rpm_Ipump))+3, 1);
  EF2_0();
}

static void BF320_231(void) {
  CF32_7();
  Temp_COsl  = read_temp( tt_p2 + 2 );
  memcpy(home->U521.U523, &Temp_COsl, 1);
  EF2_1();
}

static void BF320_232(void) {
  CF2_0();
  memcpy(home->U518.U520, ((char*)(&Temp_COsl))+1, 1);
  EF2_0();
}

static void BF320_233(void) {
  CF32_9();
  memcpy(home->U515.U517, ((char*)(&Temp_COsl))+2, 1);
  EF2_1();
}

static void BF320_234(void) {
  CF2_0();
  memcpy(home->U512.U514, ((char*)(&Temp_COsl))+3, 1);
  EF2_0();
}

static void BF320_235(void) {
  CF32_11();
  Temp_elect = read_temp( tt_p2 + 4 );
  memcpy(home->U557.U559, &Temp_elect, 1);
  EF32_11();
}

static void BF320_236(void) {
  CF2_0();
  memcpy(home->U554.U556, ((char*)(&Temp_elect))+1, 1);
  EF2_0();
}

static void BF320_237(void) {
  CF32_13();
  memcpy(home->U551.U553, ((char*)(&Temp_elect))+2, 1);
  EF2_1();
}

static void BF320_238(void) {
  CF2_0();
  memcpy(home->U548.U550, ((char*)(&Temp_elect))+3, 1);
  EF2_0();
}

static void BF320_239(void) {
  CF32_15();
  Temp_F11sl = read_temp( tt_p2 + 0 );
  memcpy(home->U509.U511, &Temp_F11sl, 1);
  EF2_1();
}

static void BF320_240(void) {
  CF2_0();
  memcpy(home->U506.U508, ((char*)(&Temp_F11sl))+1, 1);
  EF2_0();
}

static void BF320_241(void) {
  CF32_17();
  memcpy(home->U503.U505, ((char*)(&Temp_F11sl))+2, 1);
  EF2_1();
}

static void BF320_242(void) {
  CF2_0();
  memcpy(home->U500.U502, ((char*)(&Temp_F11sl))+3, 1);
  EF2_0();
}

static void BF320_243(void) {
  CF32_19();
  Temp_GasBot = read_temp12v( tt_p2 + 9 );
  memcpy(home->U581.U583, &Temp_GasBot, 1);
  EF2_1();
}

static void BF320_244(void) {
  CF2_0();
  memcpy(home->U578.U580, ((char*)(&Temp_GasBot))+1, 1);
  EF2_0();
}

static void BF320_245(void) {
  CF32_21();
  memcpy(home->U48.U51, &PresSrc_PAN, 2);
  home->U31.PresE_PAN = PresE_PAN;
  home->U43.FlowSrc_PAN = FlowSrc_PAN;
  memcpy(home->U43.U46, ((char*)(&VoltM_PAN))+3, 1);
  EF2_1();
}

static void BF320_246(void) {
  CF2_0();
  memcpy(home->U575.U577, ((char*)(&Temp_GasBot))+2, 1);
  EF2_0();
}

static void BF320_248(void) {
  CF2_0();
  memcpy(home->U572.U574, ((char*)(&Temp_GasBot))+3, 1);
  EF2_0();
}

static void BF320_249(void) {
  CF8_1();
  home->U0.FlowM_CO = FlowM_CO;
  home->U0.FlowB_CO = FlowB_CO;
  home->U0.PresBP_CO = PresBP_CO;
  memcpy(home->U0.U1, ((char*)(&PresE_CO))+3, 1);
  EF2_1();
}

static void BF320_250(void) {
  CF2_0();
  Temp_GSVbo = read_temp( tt_p2 + 3 );
  memcpy(home->U545.U547, &Temp_GSVbo, 1);
  EF2_0();
}

static void BF320_252(void) {
  CF2_0();
  memcpy(home->U542.U544, ((char*)(&Temp_GSVbo))+1, 1);
  EF2_0();
}

static void BF320_254(void) {
  CF2_0();
  memcpy(home->U539.U541, ((char*)(&Temp_GSVbo))+2, 1);
  EF2_0();
}

static void BF320_258(void) {
  CF2_0();
  memcpy(home->U536.U538, ((char*)(&Temp_GSVbo))+3, 1);
  EF2_0();
}

static void BF320_260(void) {
  CF2_0();
  Temp_PANlmp = read_temp12v( tt_p2 + 10 );
  memcpy(home->U593.U595, &Temp_PANlmp, 1);
  EF2_0();
}

static void BF320_262(void) {
  CF2_0();
  memcpy(home->U590.U592, ((char*)(&Temp_PANlmp))+1, 1);
  EF2_0();
}

static void BF320_263(void) {
  CF32_7();
  memcpy(home->U587.U589, ((char*)(&Temp_PANlmp))+2, 1);
  EF2_1();
}

static void BF320_264(void) {
  CF2_0();
  memcpy(home->U584.U586, ((char*)(&Temp_PANlmp))+3, 1);
  EF2_0();
}

static void BF320_265(void) {
  CF32_9();
  Temp_PANsl = read_temp( tt_p2 + 5 );
  memcpy(home->U533.U535, &Temp_PANsl, 1);
  EF2_1();
}

static void BF320_266(void) {
  CF2_0();
  memcpy(home->U530.U532, ((char*)(&Temp_PANsl))+1, 1);
  EF2_0();
}

static void BF320_267(void) {
  CF32_11();
  memcpy(home->U527.U529, ((char*)(&Temp_PANsl))+2, 1);
  EF32_11();
}

static void BF320_268(void) {
  CF2_0();
  memcpy(home->U524.U526, ((char*)(&Temp_PANsl))+3, 1);
  EF2_0();
}

static void BF320_269(void) {
  CF32_13();
  Temp_stack = read_temp( tt_p2 + 6 );
  memcpy(home->U569.U571, &Temp_stack, 1);
  EF2_1();
}

static void BF320_270(void) {
  CF2_0();
  memcpy(home->U566.U568, ((char*)(&Temp_stack))+1, 1);
  EF2_0();
}

static void BF320_271(void) {
  CF32_15();
  memcpy(home->U563.U565, ((char*)(&Temp_stack))+2, 1);
  EF2_1();
}

static void BF320_272(void) {
  CF2_0();
  memcpy(home->U560.U562, ((char*)(&Temp_stack))+3, 1);
  EF2_0();
}

static void BF320_273(void) {
  CF32_17();
  Trap1_re  = read_ad_value( tt_p3 + 21, 10, 10, 0 );
  memcpy(home->U185.U187, &Trap1_re, 1);
  EF2_1();
}

static void BF320_274(void) {
  CF2_0();
  memcpy(home->U182.U184, ((char*)(&Trap1_re))+1, 1);
  EF2_0();
}

static void BF320_275(void) {
  CF32_19();
  memcpy(home->U179.U181, ((char*)(&Trap1_re))+2, 1);
  EF2_1();
}

static void BF320_276(void) {
  CF2_0();
  memcpy(home->U176.U178, ((char*)(&Trap1_re))+3, 1);
  EF2_0();
}

static void BF320_277(void) {
  CF64_21();
  home->U2.FlowM_SF6 = FlowM_SF6;
  home->U6.FlowB_SF6 = FlowB_SF6;
  EF2_1();
}

static void BF320_278(void) {
  CF2_0();
  Trap1_set = read_ad_value( tt_p3 + 20, 10, 10, 0 );
  memcpy(home->U173.U175, &Trap1_set, 1);
  EF2_0();
}

static void BF320_280(void) {
  CF2_0();
  memcpy(home->U170.U172, ((char*)(&Trap1_set))+1, 1);
  EF2_0();
}

static void BF320_282(void) {
  CF2_0();
  memcpy(home->U167.U169, ((char*)(&Trap1_set))+2, 1);
  EF2_0();
}

static void BF320_284(void) {
  CF2_0();
  memcpy(home->U164.U166, ((char*)(&Trap1_set))+3, 1);
  EF2_0();
}

static void BF320_286(void) {
  CF2_0();
  Trap2_re  = read_ad_value( tt_p3 + 23, 10, 10, 0 );
  memcpy(home->U209.U211, &Trap2_re, 1);
  EF2_0();
}

static void BF320_290(void) {
  CF2_0();
  memcpy(home->U206.U208, ((char*)(&Trap2_re))+1, 1);
  EF2_0();
}

static void BF320_292(void) {
  CF2_0();
  memcpy(home->U203.U205, ((char*)(&Trap2_re))+2, 1);
  EF2_0();
}

static void BF320_294(void) {
  CF2_0();
  memcpy(home->U200.U202, ((char*)(&Trap2_re))+3, 1);
  EF2_0();
}

static void BF320_295(void) {
  CF32_7();
  Trap2_set = read_ad_value( tt_p3 + 22, 10, 10, 0 );
  memcpy(home->U197.U199, &Trap2_set, 1);
  EF2_1();
}

static void BF320_296(void) {
  CF2_0();
  memcpy(home->U194.U196, ((char*)(&Trap2_set))+1, 1);
  EF2_0();
}

static void BF320_297(void) {
  CF32_9();
  memcpy(home->U191.U193, ((char*)(&Trap2_set))+2, 1);
  EF2_1();
}

static void BF320_298(void) {
  CF2_0();
  memcpy(home->U188.U190, ((char*)(&Trap2_set))+3, 1);
  EF2_0();
}

static void BF320_299(void) {
  CF32_11();
  memcpy(home->U137.U139, &VoltBP_PAN, 1);
  EF32_11();
}

static void BF320_300(void) {
  CF2_0();
  memcpy(home->U134.U136, ((char*)(&VoltBP_PAN))+1, 1);
  EF2_0();
}

static void BF320_301(void) {
  CF32_13();
  memcpy(home->U131.U133, ((char*)(&VoltBP_PAN))+2, 1);
  EF2_1();
}

static void BF320_302(void) {
  CF2_0();
  memcpy(home->U128.U130, ((char*)(&VoltBP_PAN))+3, 1);
  EF2_0();
}

static void BF320_303(void) {
  CF32_15();
  InjState = GCstate;
  memcpy(home->U61.U63, &InjState, 1);
  EF2_1();
}

static void BF320_304(void) {
  CF2_0();
  memcpy(home->U58.U60, ((char*)(&InjState))+1, 1);
  EF2_0();
}

static void BF320_309(void) {
  CF32_21();
  home->U12.Pres1_PAN = Pres1_PAN;
  home->U2.Pres2_PAN = Pres2_PAN;
  memcpy(home->U35.U38, ((char*)(&PresSrc_PAN))+2, 2);
  EF2_1();
}

static void BF320_313(void) {
  CF8_1();
  home->U40.FlowM_F11 = FlowM_F11;
  home->U92.FlowB_F11 = FlowB_F11;
  home->U6.PresBP_F11 = PresBP_F11;
  memcpy(home->U6.U8, ((char*)(&PresE_F11))+3, 1);
  EF2_1();
}

static void (*efuncs[320])() = {
  BF32_0,
  BF32_1,
  BF320_2,
  BF32_3,
  BF320_4,
  BF32_5,
  BF320_6,
  BF320_7,
  BF320_8,
  BF320_9,
  BF320_10,
  BF320_11,
  BF320_12,
  BF320_13,
  BF320_14,
  BF320_15,
  BF320_16,
  BF320_17,
  BF320_18,
  BF320_19,
  BF320_20,
  BF320_21,
  BF320_22,
  BF64_23,
  BF320_24,
  BF64_25,
  BF320_26,
  BF64_27,
  BF320_28,
  BF64_29,
  BF320_30,
  BF64_31,
  BF32_0,
  BF32_1,
  BF320_34,
  BF32_3,
  BF320_36,
  BF32_5,
  BF320_38,
  BF320_39,
  BF320_40,
  BF320_41,
  BF320_42,
  BF320_43,
  BF320_44,
  BF320_45,
  BF320_46,
  BF320_47,
  BF320_48,
  BF320_49,
  BF320_50,
  BF320_51,
  BF320_52,
  BF320_53,
  BF320_54,
  BF64_55,
  BF320_56,
  BF320_57,
  BF320_58,
  BF64_59,
  BF320_60,
  BF64_61,
  BF320_62,
  BF64_63,
  BF32_0,
  BF32_1,
  BF320_66,
  BF32_3,
  BF320_68,
  BF32_5,
  BF320_70,
  BF320_71,
  BF320_72,
  BF320_73,
  BF320_74,
  BF320_75,
  BF320_76,
  BF320_77,
  BF320_78,
  BF320_79,
  BF320_80,
  BF320_81,
  BF320_82,
  BF320_83,
  BF320_84,
  BF320_85,
  BF320_86,
  BF64_23,
  BF320_88,
  BF64_25,
  BF320_90,
  BF64_27,
  BF320_92,
  BF64_29,
  BF320_94,
  BF64_31,
  BF32_0,
  BF32_1,
  BF320_98,
  BF32_3,
  BF320_100,
  BF32_5,
  BF320_102,
  BF320_103,
  BF320_104,
  BF320_105,
  BF320_106,
  BF320_107,
  BF320_108,
  BF320_109,
  BF320_110,
  BF320_111,
  BF320_112,
  BF320_113,
  BF320_114,
  BF320_115,
  BF320_116,
  BF320_117,
  BF320_118,
  BF64_55,
  BF320_120,
  BF320_121,
  BF320_122,
  BF64_59,
  BF320_124,
  BF64_61,
  BF320_126,
  BF64_63,
  BF32_0,
  BF32_1,
  BF320_130,
  BF32_3,
  BF320_132,
  BF32_5,
  BF320_134,
  BF320_135,
  BF320_136,
  BF320_137,
  BF320_138,
  BF320_139,
  BF320_140,
  BF320_141,
  BF320_142,
  BF320_143,
  BF320_144,
  BF320_145,
  BF320_146,
  BF320_147,
  BF320_148,
  BF320_149,
  BF320_150,
  BF64_23,
  BF320_152,
  BF64_25,
  BF320_154,
  BF64_27,
  BF320_156,
  BF64_29,
  BF320_158,
  BF64_31,
  BF32_0,
  BF32_1,
  BF320_162,
  BF32_3,
  BF320_164,
  BF32_5,
  BF320_166,
  BF320_167,
  BF320_168,
  BF320_169,
  BF320_170,
  BF320_171,
  BF320_172,
  BF320_173,
  BF320_174,
  BF320_175,
  BF320_176,
  BF320_177,
  BF320_178,
  BF320_179,
  BF320_180,
  BF320_181,
  BF320_182,
  BF64_55,
  BF320_184,
  BF320_185,
  BF320_186,
  BF64_59,
  BF320_188,
  BF64_61,
  BF320_190,
  BF64_63,
  BF32_0,
  BF32_1,
  BF320_194,
  BF32_3,
  BF320_196,
  BF32_5,
  BF320_198,
  BF320_199,
  BF320_200,
  BF320_201,
  BF320_202,
  BF320_203,
  BF320_204,
  BF320_205,
  BF320_206,
  BF320_207,
  BF320_208,
  BF320_209,
  BF320_210,
  BF320_211,
  BF320_212,
  BF320_213,
  BF320_214,
  BF64_23,
  BF320_216,
  BF64_25,
  BF320_218,
  BF64_27,
  BF320_220,
  BF64_29,
  BF320_222,
  BF64_31,
  BF32_0,
  BF32_1,
  BF320_226,
  BF32_3,
  BF320_228,
  BF32_5,
  BF320_230,
  BF320_231,
  BF320_232,
  BF320_233,
  BF320_234,
  BF320_235,
  BF320_236,
  BF320_237,
  BF320_238,
  BF320_239,
  BF320_240,
  BF320_241,
  BF320_242,
  BF320_243,
  BF320_244,
  BF320_245,
  BF320_246,
  BF64_55,
  BF320_248,
  BF320_249,
  BF320_250,
  BF64_59,
  BF320_252,
  BF64_61,
  BF320_254,
  BF64_63,
  BF32_0,
  BF32_1,
  BF320_258,
  BF32_3,
  BF320_260,
  BF32_5,
  BF320_262,
  BF320_263,
  BF320_264,
  BF320_265,
  BF320_266,
  BF320_267,
  BF320_268,
  BF320_269,
  BF320_270,
  BF320_271,
  BF320_272,
  BF320_273,
  BF320_274,
  BF320_275,
  BF320_276,
  BF320_277,
  BF320_278,
  BF64_23,
  BF320_280,
  BF64_25,
  BF320_282,
  BF64_27,
  BF320_284,
  BF64_29,
  BF320_286,
  BF64_31,
  BF32_0,
  BF32_1,
  BF320_290,
  BF32_3,
  BF320_292,
  BF32_5,
  BF320_294,
  BF320_295,
  BF320_296,
  BF320_297,
  BF320_298,
  BF320_299,
  BF320_300,
  BF320_301,
  BF320_302,
  BF320_303,
  BF320_304,
  BF32_17,
  BF2_0,
  BF32_19,
  BF2_0,
  BF320_309,
  BF2_0,
  BF64_55,
  BF2_0,
  BF320_313,
  BF2_0,
  BF64_59,
  BF2_0,
  BF64_61,
  BF2_0,
  BF64_63
};

#define TRN 320
#define TRD 1
#define LOWLIM (-2880)
#define HIGHLIM (2880)
#define NBROW 37
#define NROWMINF 1
#define NSECSPER 1
#define NROWSPER 32
#define ROLLOVER 65280
#define SYNCHVAL 0xABB4
#define NROWMAJF 320
#define MFSECNUM 32
#define MFSECDEN 1
#define SECDRIFT 90

/* Some temporary defs until everything is in the right place */
#ifndef TS_MFC_LIMIT
  #define TS_MFC_LIMIT 32767
#endif

/* This is for real: */
#ifndef DG_OTHER_CASES
  #define DG_OTHER_CASES
#endif

/* for debugging */
int check_ts = 1;

#if (NROWMAJF < 16)
  #define DP_NROWS 16
#else
  #define DP_NROWS NROWMAJF
#endif
#define NTS 3
#define incmod(x,y) if (x==((y)-1)) x = 0; else x++

static unsigned char dpdata[DP_NROWS][NBROW];
static tstamp_type tstamps[NTS], *tsps[DP_NROWS], *curts;
static unsigned short next_ts_index;
static unsigned short trans_row, col_row;
static short rowlets;
static short ts_checks;
#ifdef NEED_TIME_FUNCS
  static unsigned short CurMFC;
#endif
#define TSCHK_RTIME 1
#define TSCHK_IMPLICIT 2
#define TSCHK_CHECK 4
#define TSCHK_REQUIRED 8
static unsigned short next_minor_frame, majf_row;
#if (NROWMINF != 1)
  static unsigned short minf_row;
#endif
#ifndef DG_CUSTOM_INIT
  #define DG_CUSTOM_INIT
#endif
#ifndef TMINITFUNC
  #define TMINITFUNC
#endif
#ifndef ONINT_INIT
  #define ONINT_INIT
#endif
#ifndef ONINT_TERM
  #define ONINT_TERM
#endif
#ifndef DG_CUSTOM_TERM
  #define DG_CUSTOM_TERM
#endif


void main(int argc, char **argv) {
  oui_init_options(argc, argv);
  { int nlrsp;
	
	nlrsp = set_response(NLRSP_QUIET);
	if (find_CC(0) != -1)
	  cc_init_options(argc, argv, DCT_TM, DCT_TM, 0, 0, FORWARD_QUIT);
	set_response(nlrsp);
  }
  DG_CUSTOM_INIT;
  TMINITFUNC;
  BEGIN_MSG;
  DG_operate();
  DG_CUSTOM_TERM;
  nl_error(MSG_EXIT_NORM, "Terminated");
}


static void ts_check(void) {
  rowlets -= TRN;
  if (rowlets < LOWLIM || rowlets > HIGHLIM)
	ts_checks = TSCHK_RTIME | TSCHK_CHECK;
}

static void Collect_row(void) {
  time_t rtime, dt;
  unsigned int dmfc;
  tstamp_type *newts;
  
  #ifdef _SUBBUS_H
	tick_sic();
  #endif
  #if (NROWMINF != 1)
	if (minf_row == 0) {
  #endif
	  if ((ts_checks & TSCHK_REQUIRED) == 0 && (
	  #ifdef ROLLOVER
		  next_minor_frame == ROLLOVER ||
	  #endif
		  next_minor_frame - curts->mfc_num > TS_MFC_LIMIT))
		ts_checks |= TSCHK_IMPLICIT | TSCHK_REQUIRED;
	  if (ts_checks) {
		newts = &tstamps[next_ts_index];
		if (newts == curts)
		  nl_error(MSG_EXIT_ABNORM, "TimeStamp overflow");
		if (ts_checks & TSCHK_RTIME)
		  rtime = time(NULL);
		if (ts_checks & (TSCHK_IMPLICIT|TSCHK_CHECK)) {
		  /* mfc rate is MFSECNUM/MFSECDEN mfcs/second, so
		     mfcs/MFSECNUM = secs/MFSECDEN = dmfc
		  */
		  #if (MFSECDEN == 1)
			#define MFSECDENMUL
		  #else
			#define MFSECDENMUL * MFSECDEN
		  #endif
		  dmfc = next_minor_frame - curts->mfc_num;
		  #if (MFSECNUM != 1)
			dmfc /= MFSECNUM;
		  #endif
		  newts->secs = curts->secs + dmfc MFSECDENMUL;
		  #if (MFSECNUM != 1)
			newts->mfc_num = curts->mfc_num + dmfc*MFSECNUM;
		  #else
			newts->mfc_num = next_minor_frame;
		  #endif
		  if (ts_checks & TSCHK_CHECK) {
			/* compare rtime to newts->secs */
			#if (MFSECNUM != 1)
			  dmfc = next_minor_frame - newts->mfc_num;
			  dt = dmfc MFSECDENMUL / MFSECNUM;
			  dt = rtime - dt - newts->secs;
			#else
			  dt = rtime - newts->secs;
			#endif
			if (dt > SECDRIFT || dt < -SECDRIFT)
			  ts_checks |= TSCHK_REQUIRED;
		  }
		}
		if (ts_checks & TSCHK_REQUIRED) {
		  if (ts_checks & TSCHK_RTIME) {
			/* New time stamp:
			  If higher resolution time is available {
				f = fraction of a second extra
				rmfc = rate of mfc Hz
				f*rmfc = number of minor frames since even second.
				record floor(seconds) and mfc - f*rmfc
			  }
			*/
			newts->secs = rtime;
			newts->mfc_num = next_minor_frame;
		  }
		  #ifdef ROLLOVER
			if (next_minor_frame == ROLLOVER) {
			  next_minor_frame = 0;
			  newts->mfc_num -= ROLLOVER;
			}
		  #endif
		  incmod(next_ts_index, NTS);
		  tsps[col_row] = curts = newts;
		} else tsps[col_row] = NULL;
		ts_checks = 0;
	  } else tsps[col_row] = NULL;
	  MFCtr = next_minor_frame;
	  #ifdef NEED_TIME_FUNCS
		CurMFC = MFCtr;
	  #endif
	  next_minor_frame++;
  #if (NROWMINF != 1)
	  minf_row = 1;
    } else
	  #if (NROWMINF > 2)
		if (minf_row == NROWMINF-1)
	  #endif
  #endif
	{ /* Synch Calculations */
      #ifdef INVSYNCH
		if (majf_row == NROWMAJF-1)
		  Synch = ~SYNCHVAL;
		else
	  #else
		  Synch = SYNCHVAL;
	  #endif
  #if (NROWMINF != 1)
	  minf_row = 0;
  #endif
	}
  #if (NROWMINF > 2)
	  else ++minf_row;
  #endif
  
  /* appropriate collection function */
  home = (void *) dpdata[col_row];
  efuncs[majf_row]();
  incmod(majf_row, NROWMAJF);
  rowlets += TRD;
  
  incmod(col_row, DP_NROWS);
  if (col_row == trans_row)
	nl_error(MSG_EXIT_ABNORM, "Collection overflow");
}

/* time_prox holds information pertinent to the two timing proxies.
   If proxy == 0, the proxy has not been initialized.
   If proxy != 0, timer_type indicates what type of timer is used.
   TPTYPE_SYS means u.timer is a system timer.
   TPTYPE_TBD means u.tmrno is a timer board timer.
   TPTYPE_EXT means an external timing source is used.
*/
struct time_prox {
  pid_t proxy;
  int timer_type;
  union {
	timer_t timer;
	int tmrno;
  } u;
};
#define TPTYPE_SYS 0
#define TPTYPE_TBD 1
#define TPTYPE_EXT 2
static struct time_prox col_timer = {0,0}, ts_timer = {0,0};

#define NSEC (1000000000)

static void start_timing(int rn, int rd, struct time_prox *tp,
						 unsigned char msgtxt) {
  struct itimercb tcb;
  struct itimerspec tval, otval;

  assert(tp->proxy == 0);
  if (rn <= 4*rd) {
	set_response(0);
	tp->u.tmrno = Tmr_proxy(3, rn*(TMR_0_FREQ/rd), msgtxt);
	set_response(3);
	if (tp->u.tmrno == -1) nl_error(0, "Using System Timer");
	else {
	  tp->proxy = -1;
	  tp->timer_type = TPTYPE_TBD;
	  nl_error(0, "Using Timer Board Timer");
	  return;
	}
  }
  tp->proxy = nl_make_proxy(&msgtxt, 1);
  tcb.itcb_event.evt_value = tp->proxy;
  tp->u.timer = mktimer(TIMEOFDAY, _TNOTIFY_PROXY, &tcb);
  if (tp->u.timer == -1)
	nl_error(MSG_EXIT_ABNORM, "Error making timer: %d", tp->u.timer);
  tval.it_value.tv_sec = rn/rd;
  tval.it_value.tv_nsec = (rn%rd) * (NSEC/rd);
  tval.it_interval = tval.it_value;
  if (reltimer(tp->u.timer, &tval, &otval) == -1)
	nl_error(MSG_EXIT_ABNORM, "Error in reltimer: %d\n", errno);
  tp->timer_type = TPTYPE_SYS;
}

/* Only detaches proxy if timing is internal */
static void stop_timing(struct time_prox *tp) {
  if (tp->proxy != 0) {
	switch (tp->timer_type) {
	  case TPTYPE_TBD:
		Tmr_reset(tp->u.tmrno);
		tp->proxy = 0;
		break;
	  case TPTYPE_SYS:
		rmtimer(tp->u.timer);
		tp->u.timer = 0;
		if (tp->proxy != 0) {
		  qnx_proxy_detach(tp->proxy);
		  tp->proxy = 0;
		}
		break;
	  default:
		break;
	}
  }
}

#define MKDCTV(x,y) ((x<<8)|y)
#define DCTV(x,y) MKDCTV(DCT_##x,DCV_##y)

int DG_DASCmd(unsigned char type, unsigned char val) {
  switch (MKDCTV(type, val)) {
	case DCTV(TM,TM_START):
	  if (check_ts)
		start_timing(10, 1, &ts_timer, COL_TSPROXY);
	  if (col_timer.proxy == 0)
		start_timing(tmi(nsecsper), tmi(nrowsper), &col_timer, COL_REG_SIG);
	  ts_checks = TSCHK_RTIME | TSCHK_REQUIRED;
	  next_ts_index = 0;
	  curts = NULL;
	  trans_row = col_row = 0;
	  rowlets = 0;
	  next_minor_frame = majf_row = 0;
	  #if (NROWMINF != 1)
		minf_row = 0;
	  #endif
	  ONINT_INIT
	  break;
	case DCTV(TM,TM_END):
	case DCTV(QUIT,QUIT):
	  #ifdef _SUBBUS_H
		disarm_sic();
	  #endif
	  ONINT_TERM
	  stop_timing(&col_timer);
	  stop_timing(&ts_timer);
	  break;
	default:
	  break;
  }
  return(0);
}


#include <sys/seginfo.h>

/* I've added some verbosity here, but I'd like to add more:
   The messages should include the pointer ID, which we don't know here.
   I'd have to change tmc together with this file...
*/
static void COL_get_pointer(pid_t sent_tid, void far **pptr, void far *ptr) {
  unsigned segt;
  
  if (*pptr == &nullregion) {
	segt = qnx_segment_get(sent_tid, FP_SEG(ptr), 0);
	if (segt != -1) {
	  *pptr = MK_FP(segt, FP_OFF(ptr));
	  reply_byte(sent_tid, DAS_OK);
	  nl_error(0, "Pointer set from pid %d", sent_tid);
	  return;
	} else nl_error(MSG_WARN, "Pointer failed from pid %d", sent_tid);
  } else nl_error(MSG_WARN, "Pointer from pid %d previously set", sent_tid);
  reply_byte(sent_tid, DAS_UNKN);
}

static void COL_free_pointer(pid_t sent_tid, void far **pptr) {
  if (*pptr != &nullregion) {
	qnx_segment_free( FP_SEG(*pptr));
	*pptr = &nullregion;
	reply_byte(sent_tid, DAS_OK);
  } else reply_byte(sent_tid, DAS_UNKN);
}

static void COL_recv_proxy(pid_t sent_tid, pid_t *prx, pid_t proxy) {
  if (*prx == 0) {
	*prx = proxy;
	reply_byte(sent_tid, DAS_OK);
  } else reply_byte(sent_tid, DAS_UNKN);
}

static void COL_end_proxy(pid_t sent_tid, pid_t *prx, struct colmsg *c) {
  c->u.proxy = *prx;
  c->type = *prx == 0 ? DAS_UNKN : DAS_OK;
  *prx = 0;
  Reply(sent_tid, c, sizeof(struct colmsg));
}

#include <stddef.h>

static void read_col_send( void *dest, size_t length,
					struct colmsg *cmsg, pid_t sent_tid ) {
  size_t trulen = min( length, cmsg->u.data.size );
  if ( trulen <= MAX_COLMSG_SIZE-5 ) {
	memcpy( dest, cmsg->u.data.data, trulen );
  } else {
	Readmsg( sent_tid,
	  offsetof( struct colmsg, u.data.data ), dest, trulen );
  }
}

int DG_other(unsigned char *msg_ptr, int sent_tid) {
  struct colmsg *cmsg;
  
  switch (*msg_ptr) {
	case COL_REG_SIG:
	  if (dbr_info.tm_started) Collect_row();
	  break;
	case COL_TSPROXY:
	  if (dbr_info.tm_started) ts_check();
	  break;
	case COL_REGULATE:
	  {
		struct dgreg_rep dgregrep;
		unsigned char msgtxt = COL_REG_SIG;

		if (col_timer.proxy != 0) {
		  dgregrep.reply_code = DAS_BUSY;
		  dgregrep.proxy = col_timer.proxy;
		} else {
		  col_timer.proxy = nl_make_proxy(&msgtxt, 1);
		  col_timer.timer_type = TPTYPE_EXT;
		  dgregrep.proxy = col_timer.proxy;
		  dgregrep.reply_code = DAS_OK;
		}
		Reply(sent_tid, &dgregrep, sizeof(dgregrep));
	  }
	  return(0);
	case COL_SET_POINTER:
	  cmsg = (struct colmsg *)msg_ptr;
	  switch (cmsg->id) {
		case 5: COL_get_pointer(sent_tid, &MSDstatePtr, cmsg->u.pointer); break;
		case 4: COL_get_pointer(sent_tid, &GCstatePtr, cmsg->u.pointer); break;
		default: return(reply_byte(sent_tid, DAS_UNKN));
	  }
	  return(0);
	case COL_RESET_POINTER:
	  cmsg = (struct colmsg *)msg_ptr;
	  switch (cmsg->id) {
		case 5: COL_free_pointer(sent_tid, &MSDstatePtr); break;
		case 4: COL_free_pointer(sent_tid, &GCstatePtr); break;
		default: return(reply_byte(sent_tid, DAS_UNKN));
	  }
	  return(0);
	case COL_SET_PROXY:
	  cmsg = (struct colmsg *)msg_ptr;
	  switch (cmsg->id) {
		case 12: COL_recv_proxy(sent_tid, &maycomm_proxy, cmsg->u.proxy); break;
		case 7: COL_recv_proxy(sent_tid, &MSDeng_proxy, cmsg->u.proxy); break;
		case 6: COL_recv_proxy(sent_tid, &MSDresp_proxy, cmsg->u.proxy); break;
		case 4: COL_recv_proxy(sent_tid, &valco_proxy, cmsg->u.proxy); break;
		case 8: COL_recv_proxy(sent_tid, &ECDresp_proxy, cmsg->u.proxy); break;
		case 3: COL_recv_proxy(sent_tid, &rs485_proxy, cmsg->u.proxy); break;
		default: return(reply_byte(sent_tid,DAS_UNKN));
	  }
	  return(0);
	case COL_RESET_PROXY:
	  cmsg = (struct colmsg *)msg_ptr;
	  switch (cmsg->id) {
		case 12: COL_end_proxy(sent_tid, &maycomm_proxy, cmsg); break;
		case 7: COL_end_proxy(sent_tid, &MSDeng_proxy, cmsg); break;
		case 6: COL_end_proxy(sent_tid, &MSDresp_proxy, cmsg); break;
		case 4: COL_end_proxy(sent_tid, &valco_proxy, cmsg); break;
		case 8: COL_end_proxy(sent_tid, &ECDresp_proxy, cmsg); break;
		case 3: COL_end_proxy(sent_tid, &rs485_proxy, cmsg); break;
		default: return(reply_byte(sent_tid,DAS_UNKN));
	  }
	  return(0);
	case COL_SEND:
	  cmsg = (struct colmsg *)msg_ptr;
	  switch (cmsg->id) {
		case COL_SEND_INIT:
		  if (stricmp(cmsg->u.name, "tdl") == 0) {
			cmsg->u.data.id = 6;
			cmsg->u.data.size = sizeof(tdl);
		  } else if (stricmp(cmsg->u.name, "MSDeng") == 0) {
			cmsg->u.data.id = 5;
			cmsg->u.data.size = sizeof(MSDeng);
		  } else if (stricmp(cmsg->u.name, "MSDresp") == 0) {
			cmsg->u.data.id = 4;
			cmsg->u.data.size = sizeof(MSDresp);
		  } else if (stricmp(cmsg->u.name, "valvePos") == 0) {
			cmsg->u.data.id = 3;
			cmsg->u.data.size = sizeof(valvePos);
		  } else if (stricmp(cmsg->u.name, "ECDresp") == 0) {
			cmsg->u.data.id = 2;
			cmsg->u.data.size = sizeof(ECDresp);
		  } else if (stricmp(cmsg->u.name, "Omega485") == 0) {
			cmsg->u.data.id = 1;
			cmsg->u.data.size = sizeof(Omega485);
		  } else return reply_byte(sent_tid,DAS_UNKN);
		  cmsg->type = DAS_OK;
		  Reply(sent_tid, cmsg, offsetof(struct colmsg, u.data.data));
		  return 0;
		case COL_SEND_SEND:
		  switch (cmsg->u.data.id) {
			case 6:
			  read_col_send(&tdl, sizeof(tdl), cmsg, sent_tid );
			  break;
			case 5:
			  read_col_send(&MSDeng, sizeof(MSDeng), cmsg, sent_tid );
			  break;
			case 4:
			  read_col_send(&MSDresp, sizeof(MSDresp), cmsg, sent_tid );
			  break;
			case 3:
			  read_col_send(&valvePos, sizeof(valvePos), cmsg, sent_tid );
			  break;
			case 2:
			  read_col_send(&ECDresp, sizeof(ECDresp), cmsg, sent_tid );
			  break;
			case 1:
			  read_col_send(&Omega485, sizeof(Omega485), cmsg, sent_tid );
			  break;
			default: return reply_byte(sent_tid, DAS_UNKN);
		  }
		  break;
		case COL_SEND_RESET: break;
		default: return reply_byte(sent_tid,DAS_UNKN);
	  }
	  return reply_byte(sent_tid, DAS_OK);
	DG_OTHER_CASES /* defined via preprocessor elsewhere */
	default:
	  return(reply_byte(sent_tid, DAS_UNKN));
  }
  /* We only get here on the two proxy calls: proxies don't need reply */
  return(0);
}

/* Need a 'current time stamp' different from the one in dbr_info,
   since that one pertains to stuff already being transmitted.
   Need to optimize for wrap-around when DG_s_data is augmented
   to support two blocks of data.
*/
int DG_get_data(unsigned int n_rows) {
  int rowlim, i;
  
  if (trans_row != col_row) {
	if (tsps[trans_row] != NULL) {
	  DG_s_tstamp(tsps[trans_row]);
	  tsps[trans_row] = NULL;
	} else {
	  rowlim = trans_row + n_rows;
	  if (DP_NROWS < rowlim) rowlim = DP_NROWS;
	  if (col_row > trans_row && col_row < rowlim) rowlim = col_row;
	  for (i = trans_row; i < rowlim && tsps[i] == NULL; i++);
	  assert(i != trans_row);
	  DG_s_data(i-trans_row, dpdata[trans_row], 0, NULL);
	  trans_row = i;
	  if (trans_row == DP_NROWS) trans_row = 0;
	}
  }
  return(0);
}

#ifdef NEED_TIME_FUNCS
  #if (NROWMINF != 1)
	#define ROWS(x) (((unsigned long)(x))*NROWMINF+minf_row)
  #else
	#define ROWS(x) (x)
  #endif
  #if (NSECSPER != 1)
	#define FRACSECS(x) (((unsigned long)ROWS(x))*NSECSPER)
  #else
	#define FRACSECS(x) ROWS(x)
  #endif
  long itime(void) {
	  return(dbr_info.t_stmp.secs +
		FRACSECS(CurMFC-dbr_info.t_stmp.mfc_num)
	#if (NROWSPER != 1)
		  / NROWSPER
	#endif
		  );
  }
  double dtime(void) {
	  return(dbr_info.t_stmp.secs +
		(double) FRACSECS(CurMFC-dbr_info.t_stmp.mfc_num)
	#if (NROWSPER != 1)
		  / NROWSPER
	#endif
		  );
  }
  double etime(void) {
	double t;
	static double t0 = -1e9;
	
	t = dtime();
	if (t0 == -1e9) t0 = t;
	return(t - t0);
  }
#endif

dbr_info_type dbr_info = {
  "", /* tmid */
  37, /* NBMINF */
  37, /* NBROW */
  320, /* NROWSMAJF */
  1, /* NSECSPER */
  32, /* NROWSPER */
  0, 1, /* MFC lsb col, msb col */
  0xABB4, /* Synch Value */
  0 /* not inverted */
};
